/***************************************************************************
                          account.cpp  -  description
                             -------------------
    begin                : Sun Jan 5 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "account.h"

#include "contact/msnobject.h"
#include "utils/kmessconfig.h"
#include "accountsmanager.h"
#include "kmessdebug.h"

#include <QDir>
#include <QRegExp>

#include <KConfigGroup>

#include <KApplication>
#include <KCodecs>
#include <KGlobal>
#include <KLocale>
#include <KStandardDirs>


#ifdef KMESSDEBUG_ACCOUNT
  #define KMESSDEBUG_ACCOUNT_GENERAL
  #define KMESSDEBUG_ACCOUNT_DIRTY
#endif



// The constructor
Account::Account()
 : autoreplyMessage_( i18n("I'm away from my computer") ),
   chatStyle_( "Fresh" ),
   contactFontColor_("#000000"),
   deleted_(false),
   dirty_(false),
   doNotifyContactsOnline_(true),
   doNotifyContactsStatus_(false),
   doNotifyContactsChatStart_(true),
   doNotifyContactsChatMessages_(false),
   doNotifyContactsOffline_(true),
   doNotifyInboxEmails_(true),
   doNotifyOtherEmails_(false),
   emailSupported_(true),  // Assume true (for new accounts), until the msn server proves the opposite
   emoticonStyle_("KMess-new"),
   fontColor_("#000000"),
   friendlyName_( i18n("Your name"), false ), // Do not parse this name, CurrentAccount is not ready when initializing
   guestAccount_(true),
   groupFollowupMessages_(true),
   handle_( i18n("you@hotmail.com") ),
   idleTime_(5),
   initialStatus_(STATUS_ONLINE),
   listPictureViewMode_(0),
   notificationDuration_(10),
   saveChats_(false),
   savedChatDirectoryStructure_( 0 ),
   savePassword_(false),
   shakeNudge_(true),
   showAllowedContacts_(false),
   showContactsByGroup_(true),
   showEmail_(true),
   showMessageTime_(true),
   showNowListening_(false),
   showOfflineContacts_(true),
   showOtherFolders_(true),
   showRemovedContacts_(false),
   showSidebar_(true),
   tabbedChatMode_( 0 ),
   useAutologin_(false),
   useContactFont_(false),
   useEmoticons_(true),
   useFontEffects_(true),
   useIdleTimer_(true),
   useListFormatting_(false),
   useChatFormatting_(false),

   verified_(true),

   showPicture_(true),
   msnObject_(0)
{
  QDir homeDir;

  font_.setFamily("Arial");
  font_.setBold(false);
  font_.setItalic(false);
  font_.setUnderline(false);
  font_.setPointSize(12);

  contactFont_.setFamily("Arial");
  contactFont_.setBold(false);
  contactFont_.setItalic(false);
  contactFont_.setUnderline(false);

  homeDir = QDir::home();
  saveChatPath_ = homeDir.absolutePath();
}



// The destructor
Account::~Account()
{
  if ( msnObject_ )
    delete msnObject_;

#ifdef KMESSDEBUG_ACCOUNT_GENERAL
  kDebug() << "DESTROYED. [handle=" << handle_ << "]";
#endif
}



// Copy an account
void Account::copyAccount( const Account *account )
{
  // This method is called to write information from a global Account
  // object to the CurrentAccount and visa-versa.
  // See also: copyAccountUISettings() for properties only changed by the GUI.

  setListPictureViewMode     ( account->getListPictureViewMode() );
  setLoginInformation        ( account->getHandle(),
                               account->getFriendlyName( STRING_ORIGINAL ),
                               account->getPassword() );
  setShowAllowedContacts     ( account->getShowAllowedContacts() );
  setShowContactsByGroup     ( account->getShowContactsByGroup() );
  setShowOfflineContacts     ( account->getShowOfflineContacts() );
  setShowRemovedContacts     ( account->getShowRemovedContacts() );
  setEmoticonStyle           ( account->getEmoticonStyle()       );

  setAlertsInformation       ( account->getUseIdleTimer(),
                               account->getIdleTime(),
                               account->getShakeNudge(),
                               account->getShowNowListening() );
  setNotificationsInformation( account->getNotifyContactsOnline(),
                               account->getNotifyContactsStatus(),
                               account->getNotifyContactsChatStart(),
                               account->getNotifyContactsChatMessages(),
                               account->getNotifyContactsOffline(),
                               account->getNotifyInboxEmails(),
                               account->getNotifyOtherEmails(),
                               account->getNotificationDuration() );
  setChatInformation         ( account->getUseContactFont(),
                               account->getUseEmoticons(),
                               account->getUseFontEffects(),
                               account->getUseListFormatting(),
                               account->getUseChatFormatting(),
                               account->getShowMessageTime(),
                               account->getGroupFollowupMessages(),
                               account->getTabbedChatMode(),
                               account->getChatStyle()                     );
  setChatLoggingInformation  ( account->getSaveChats(),
                               account->getSaveChatPath(),
                               account->getSavedChatDirectoryStructure()   );
  setEmailInformation        ( account->getShowEmail(),
                               account->getShowOtherFolders()              );
  setEmailSupported          ( account->getEmailSupported()                );
  setFontInformation         ( account->getFont(),
                               account->getFontColor(),
                               account->getContactFont(),
                               account->getContactFontColor()              );
  setPersonalMessage         ( account->getPersonalMessage( STRING_ORIGINAL ) );

  setUseAutologin            ( account->getUseAutologin()                  );
  setAutoreplyMessage        ( account->getAutoreplyMessage()              );
  setShowSidebar             ( account->getShowSidebar()                   );
  setInitialStatus           ( account->getInitialStatus()                 );
  setGuestAccount            ( account->isGuestAccount()                   );

  setShowPicture             ( account->getShowPicture()                   );
  setTemporaryPassword       ( account->getTemporaryPassword()             );
  setVerified                ( account->isVerified()                       );

  dirty_              = account->isDirty();
  displayPicturePath_ = account->displayPicturePath_;
  updateMsnObject();
}



// Copy an account's user-interface, not settings, changable properties
void Account::copyAccountUISettings( const Account *account )
{
  // All properties that are modified through the main gui are listed here.

  // Contact list menu's
  setListPictureViewMode( account->getListPictureViewMode() );
  setShowAllowedContacts( account->getShowAllowedContacts() );
  setShowOfflineContacts( account->getShowOfflineContacts() );
  setShowRemovedContacts( account->getShowRemovedContacts() );
  setShowContactsByGroup( account->getShowContactsByGroup() );

  // Contact list header
  setPersonalMessage    ( account->getPersonalMessage( STRING_ORIGINAL ) );
  setAutoreplyMessage   ( account->getAutoreplyMessage()    );

  // Login
  setGuestAccount       ( account->isGuestAccount()         );
  setEmailSupported     ( account->getEmailSupported()      );
  setVerified           ( account->isVerified()             );
  password_             = account->getPassword();

  // Chat settings
  setShowSidebar        ( account->getShowSidebar()         );
  setFontInformation    ( account->getFont(),
                          account->getFontColor(),
                          account->getContactFont(),
                          account->getContactFontColor()    );


  // Settings
  displayPicturePath_ = account->displayPicturePath_;
  dirty_              = account->isDirty();
}



// Set the account as deleted
void Account::setDeleted()
{
  // Can be called twice, avoid warnings.
  if( deleted_ )
  {
    return;
  }

  // Mark as deleted, so other classes avoid saving the settings again.
  deleted_      = true;
  guestAccount_ = true;
}



// Read the auto-reply message.
const QString& Account::getAutoreplyMessage() const
{
  return autoreplyMessage_;
}



// Return the chat style name.
const QString& Account::getChatStyle() const
{
  return chatStyle_;
}



// Return the font used for contact messages, if forced to.
const QFont& Account::getContactFont() const
{
  return contactFont_;
}



// Return the color of the forced contact font.
const QString& Account::getContactFontColor() const
{
  return contactFontColor_;
}



// Return whether email notifications are supported
bool Account::getEmailSupported() const
{
  return emailSupported_;
}


// Return the user message font.
const QFont& Account::getFont() const
{
  return font_;
}



// Return the color of the user message font.
const QString& Account::getFontColor() const
{
  return fontColor_;
}



// Return the user's friendlyName
QString Account::getFriendlyName( FormattingMode mode ) const
{
  return friendlyName_.getString( mode );
}



// Whether follow-up messages from the contact should be grouped.
bool Account::getGroupFollowupMessages() const
{
  return groupFollowupMessages_;
}



// Return the user's handle
QString Account::getHandle() const
{
  return handle_;
}



// Return the length of time waited before the user goes idle.
const int& Account::getIdleTime() const
{
  return idleTime_;
}



// Return the time popup windows are shown.
const int& Account::getNotificationDuration() const
{
  return notificationDuration_;
}



// Return if popup notifications will be shown for contacts which come online.
bool Account::getNotifyContactsOnline() const
{
  return doNotifyContactsOnline_;
}



// Return if popup notifications will be shown for contacts which change their msn status.
bool Account::getNotifyContactsStatus() const
{
  return doNotifyContactsStatus_;
}



// Return if popup notifications will be shown for contacts which start a chat with you.
bool Account::getNotifyContactsChatStart() const
{
  return doNotifyContactsChatStart_;
}



// Return if popup notifications will be shown for all received chat messages.
bool Account::getNotifyContactsChatMessages() const
{
  return doNotifyContactsChatMessages_;
}



// Return if popup notifications will be shown for contacts which go offline.
bool Account::getNotifyContactsOffline() const
{
  return doNotifyContactsOffline_;
}



// Return if popup notifications will be shown for mail incoming in the Inbox.
bool Account::getNotifyInboxEmails() const
{
  return doNotifyInboxEmails_;
}



// Return if popup notifications will be shown for mail incoming in other folders.
bool Account::getNotifyOtherEmails() const
{
  return doNotifyOtherEmails_;
}



// Return whether now listening information should be displayed.
bool Account::getShowNowListening() const
{
  return showNowListening_;
}



// Return the user's password
QString Account::getPassword() const
{
  return password_;
}



// Return the personal message
QString Account::getPersonalMessage( FormattingMode mode ) const
{
  return personalMessage_.getString( mode );
}



// Return the selected emoticon's style
const QString& Account::getEmoticonStyle() const
{
  return emoticonStyle_;
}



// Return the status to set upon connection
Status Account::getInitialStatus() const
{
  return initialStatus_;
}



// Return whether or not to save chats.
bool Account::getSaveChats() const
{
  return saveChats_;
}



// Return the base directory to which chats are saved.
const QString& Account::getSaveChatPath() const
{
  return saveChatPath_;
}



// Return the code of the structure used to save chats.
const int& Account::getSavedChatDirectoryStructure() const
{
  return savedChatDirectoryStructure_;
}



// Read whether the password must be stored
bool Account::getSavePassword() const
{
  return savePassword_;
}



// Read whether the chat window should shake when a nudge is received or sent.
bool Account::getShakeNudge() const
{
  return shakeNudge_;
}



// Read whether or not allowed contacts are shown.
bool Account::getShowAllowedContacts() const
{
  return showAllowedContacts_;
}



// Read whether contacts are shown by group or by online/offline.
bool Account::getShowContactsByGroup() const
{
  return showContactsByGroup_;
}



// Read whether the email information should be shown in the main view.
bool Account::getShowEmail() const
{
  return showEmail_;
}



// Read whether message times should be shown.
bool Account::getShowMessageTime() const
{
  return showMessageTime_;
}



// Read whether offline contacts should be shown.
bool Account::getShowOfflineContacts() const
{
  return showOfflineContacts_;
}



// Read property of bool showOtherFolders.
bool Account::getShowOtherFolders() const
{
  return showOtherFolders_;
}



// Read whether or not removed (reverse) contacts are shown.
bool Account::getShowRemovedContacts() const
{
  return showRemovedContacts_;
}



// Read whether or not the sidebar should be shown in chats
bool Account::getShowSidebar() const
{
  return showSidebar_;
}



// Read the way the chats will be grouped together
int Account::getTabbedChatMode() const
{
  return tabbedChatMode_;
}



// Return the a temporary password for the account for login attempts from the initialview dialog
const QString & Account::getTemporaryPassword() const
{
  return temporaryPassword_;
}



// Return whether or not to autologin with this profile.
bool Account::getUseAutologin() const
{
  return useAutologin_;
}



// Return whether or not to enable MSN Plus! formatting in chat
bool Account::getUseChatFormatting() const
{
  return useChatFormatting_;
}



// Return whether or not to show contact messages in the stored font.
bool Account::getUseContactFont() const
{
  return useContactFont_;
}



// Return whether or not to show emoticons in chats.
bool Account::getUseEmoticons() const
{
  return useEmoticons_;
}



// Return whether or not to use the bold/italic/underline effects.
bool Account::getUseFontEffects() const
{
  return useFontEffects_;
}



// Return whether the idle timer should be used.
bool Account::getUseIdleTimer() const
{
  return useIdleTimer_;
}



// Return whether or not to enable MSN Plus! formatting in the contact list
bool Account::getUseListFormatting() const
{
  return useListFormatting_;
}



// Returns whether or not to send our display picture
bool Account::getShowPicture() const
{
  return showPicture_;
}



// Returns display mode used to show pictures in contact list.
int Account::getListPictureViewMode() const
{
  return listPictureViewMode_;
}



// Return the path to the current display picture
const QString Account::getPicturePath() const
{
  // Return the picture path if the user has selected to show it; else return an empty string
  if( ! showPicture_ )
  {
    return QString();
  }

  // if the user has selected a custom picture, it will be located in
  // ~/.kde/share/apps/kmess/<account@handle>/displaypics/<pictureHash>.png

  // If the file can't be found, fallback to the default picture
  if( displayPicturePath_.isEmpty() || ! QFile::exists( displayPicturePath_ ) )
  {
    return KGlobal::dirs()->findResource( "data", "kmess/pics/kmesspic.png" );
  }

  return displayPicturePath_;
}



// Returns the string representation of the MsnObject
const QString Account::getMsnObjectString() const
{
  if ( msnObject_ )
    return msnObject_->objectString();
  else
    return "";
}



// Whether the account is marked to be deleted
bool Account::isDeleted() const
{
  return deleted_;
}



// Whether the account has unsaved settings changes
bool Account::isDirty() const
{
  return dirty_;
}



// Whether the account is a guest account, not permanently saved on the computer
bool Account::isGuestAccount() const
{
  return guestAccount_;
}



// Validate a contact email
bool Account::isValidEmail( QString email )
{
  return email.contains( QRegExp( "^[A-Z0-9._%\\-]+@(?:[A-Z0-9\\-]+\\.)+[A-Z]{2,4}$", Qt::CaseInsensitive ) );
}



// Return whether the account's passport is verified
bool Account::isVerified() const
{
  return verified_;
}



// Read in account properties
void Account::readProperties( const QString &handle )
{
  handle_ = handle;
  QDir    homeDir;
  QString defaultPath;

  homeDir = QDir::home();
  defaultPath = homeDir.absolutePath();

  // Choose the main group
  KConfigGroup profileGroup = KMessConfig::instance()->getAccountConfig( handle_, "Account" );

  autoreplyMessage_              = profileGroup.readEntry( "autoreplymessage",             i18n("I'm away from my computer") );
  chatStyle_                     = profileGroup.readEntry( "chatStyle",                    "Fresh"     );
  contactFont_.setFamily         ( profileGroup.readEntry( "contactfontfamily",            "Arial"     ) );
  contactFont_.setBold           ( profileGroup.readEntry( "contactfontbold",              false       ) );
  contactFont_.setItalic         ( profileGroup.readEntry( "contactfontitalic",            false       ) );
  contactFont_.setUnderline      ( profileGroup.readEntry( "contactfontunderline",         false       ) );
  contactFont_.setPointSize      ( profileGroup.readEntry( "contactfontpointsize",         10          ) );
  contactFontColor_              = profileGroup.readEntry( "contactfontcolor" ,            "#000000"   );
  displayPicturePath_            = profileGroup.readEntry( "displayPicturePath",           ""          );
  doNotifyContactsOnline_        = profileGroup.readEntry( "doNotifyContactsOnline",       true        );
  doNotifyContactsStatus_        = profileGroup.readEntry( "doNotifyContactsStatus",       false       );
  doNotifyContactsChatStart_     = profileGroup.readEntry( "doNotifyContactsChatStart",    true        );
  doNotifyContactsChatMessages_  = profileGroup.readEntry( "doNotifyContactsChatMessages", false       );
  doNotifyContactsOffline_       = profileGroup.readEntry( "doNotifyContactsOffline",      true        );
  doNotifyInboxEmails_           = profileGroup.readEntry( "doNotifyInboxEmails",          true        );
  doNotifyOtherEmails_           = profileGroup.readEntry( "doNotifyOtherEmails",          false       );
  emailSupported_                = profileGroup.readEntry( "emailSupported",               true        );
  emoticonStyle_                 = profileGroup.readEntry( "emoticonStyle",                "KMess-new" );
  font_.setFamily                ( profileGroup.readEntry( "fontfamily",                   "Arial"     ) );
  font_.setBold                  ( profileGroup.readEntry( "fontbold",                     false       ) );
  font_.setItalic                ( profileGroup.readEntry( "fontitalic",                   false       ) );
  font_.setPointSize             ( profileGroup.readEntry( "fontpointsize",                12          ) );
  font_.setUnderline             ( profileGroup.readEntry( "fontunderline",                false       ) );
  fontColor_                     = profileGroup.readEntry( "fontColor" ,                   "#000000"   );
  friendlyName_.setString        ( profileGroup.readEntry( "friendlyName" ,                ""          ) );
  groupFollowupMessages_         = profileGroup.readEntry( "groupfollowupmessages",        true        );
  idleTime_                      = profileGroup.readEntry( "idleTime",                     5           );
  initialStatus_         = (Status)profileGroup.readEntry( "initialStatus",          (int) STATUS_ONLINE);
  listPictureViewMode_           = profileGroup.readEntry( "listPictureViewMode",          0           );
  notificationDuration_          = profileGroup.readEntry( "notification",                 5           );
  personalMessage_.setString     ( profileGroup.readEntry( "personalMessage",              ""          ) );
  saveChatPath_                  = profileGroup.readEntry( "saveChatPath",                 defaultPath );
  saveChats_                     = profileGroup.readEntry( "saveChats",                    false       );
  savedChatDirectoryStructure_   = profileGroup.readEntry( "savedChatDirectoryStructure",  0           );
  savePassword_                  = profileGroup.readEntry( "savePassword",                 false       );
  shakeNudge_                    = profileGroup.readEntry( "shakeNudge",                   true        );
  showAllowedContacts_           = profileGroup.readEntry( "showAllowed",                  false       );
  showContactsByGroup_           = profileGroup.readEntry( "showByGroup",                  true        );
  showEmail_                     = profileGroup.readEntry( "showEmail",                    true        );
  showPicture_                   = profileGroup.readEntry( "showPicture",                  true        );
  showMessageTime_               = profileGroup.readEntry( "showMessageTime",              true        );
  showNowListening_              = profileGroup.readEntry( "showNowListening",             false       );
  showOfflineContacts_           = profileGroup.readEntry( "showOfflineContacts",          true        );
  showOtherFolders_              = profileGroup.readEntry( "showOtherFolders",             true        );
  showRemovedContacts_           = profileGroup.readEntry( "showRemoved",                  false       );
  showSidebar_                   = profileGroup.readEntry( "showSidebar",                  true        );
  tabbedChatMode_                = profileGroup.readEntry( "tabbedChatMode",               2           );
  useAutologin_                  = profileGroup.readEntry( "useAutologin",                 false       );
  useChatFormatting_             = profileGroup.readEntry( "useChatFormatting",            false       );
  useContactFont_                = profileGroup.readEntry( "useContactFont",               false       );
  useEmoticons_                  = profileGroup.readEntry( "useEmoticons",                 true        );
  useFontEffects_                = profileGroup.readEntry( "useFontEffects",               true        );
  useIdleTimer_                  = profileGroup.readEntry( "useIdleTimer",                 true        );
  useListFormatting_             = profileGroup.readEntry( "useListFormatting",            false       );

  updateMsnObject();

  // After reading options from file, consider "dirty" to still be false
  guestAccount_ = false;
  dirty_        = false;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  kDebug() << "Resetting 'dirty' to false.";
#endif

  // If a chat style is removed, fall back to the default.
  // This default is "Fresh" as of 1.5 final, to improve the first impression a user gets.
  // The previous "Default" theme has been renamed to "Classic".
  // This code also makes the migration from 1.5-pre2 to 1.5 (and beyond) easier.
  if( KGlobal::dirs()->findResource( "data", "kmess/styles/" + chatStyle_ + "/" + chatStyle_ + ".xsl" ).isEmpty() )
  {
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Auto-corrected chat & setting, setting 'dirty' to true.";
#endif

    // Fall back to our new default.
    chatStyle_ = "Fresh";
    groupFollowupMessages_ = true;  // Make sure Fresh looks good.
    dirty_ = true;
  }
}



// Save account properties
void Account::saveProperties()
{
  // Protect against unwanted usage
  if( guestAccount_ )
  {
    kWarning() << "Application attempted to save a guest account!";
  }
  if( deleted_ )
  {
    kWarning() << "Application attempted to save the deleted account again!";
  }

  // Choose the main group
  KConfigGroup profileGroup = KMessConfig::instance()->getAccountConfig( handle_, "Account" );

  if ( dirty_ )
  {
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Account " << handle_ << " is 'dirty', save properties.";
#endif

    // Write new settings
    profileGroup.writeEntry( "autoreplymessage",             autoreplyMessage_             );
    profileGroup.writeEntry( "chatStyle",                    chatStyle_                    );
    profileGroup.writeEntry( "contactfontfamily",            contactFont_.family()         );
    profileGroup.writeEntry( "contactfontbold",              contactFont_.bold()           );
    profileGroup.writeEntry( "contactfontitalic",            contactFont_.italic()         );
    profileGroup.writeEntry( "contactfontunderline",         contactFont_.underline()      );
    profileGroup.writeEntry( "contactfontpointsize",         contactFont_.pointSize()      );
    profileGroup.writeEntry( "contactfontcolor",             contactFontColor_             );
    profileGroup.writeEntry( "displayPicturePath",           displayPicturePath_           );
    profileGroup.writeEntry( "doNotifyContactsOnline",       doNotifyContactsOnline_       );
    profileGroup.writeEntry( "doNotifyContactsStatus",       doNotifyContactsStatus_       );
    profileGroup.writeEntry( "doNotifyContactsChatStart",    doNotifyContactsChatStart_    );
    profileGroup.writeEntry( "doNotifyContactsChatMessages", doNotifyContactsChatMessages_ );
    profileGroup.writeEntry( "doNotifyContactsOffline",      doNotifyContactsOffline_      );
    profileGroup.writeEntry( "doNotifyInboxEmails",          doNotifyInboxEmails_          );
    profileGroup.writeEntry( "doNotifyOtherEmails",          doNotifyOtherEmails_          );
    profileGroup.writeEntry( "emailSupported",               emailSupported_               );
    profileGroup.writeEntry( "emoticonStyle",                emoticonStyle_                );
    profileGroup.writeEntry( "fontfamily",                   font_.family()                );
    profileGroup.writeEntry( "fontbold",                     font_.bold()                  );
    profileGroup.writeEntry( "fontitalic",                   font_.italic()                );
    profileGroup.writeEntry( "fontunderline",                font_.underline()             );
    profileGroup.writeEntry( "fontpointsize",                font_.pointSize()             );
    profileGroup.writeEntry( "fontColor",                    fontColor_                    );
    profileGroup.writeEntry( "friendlyName",                 friendlyName_.getOriginal()   );
    profileGroup.writeEntry( "groupfollowupmessages",        groupFollowupMessages_        );
    profileGroup.writeEntry( "idleTime",                     idleTime_                     );
    profileGroup.writeEntry( "listPictureViewMode",          listPictureViewMode_          );
    profileGroup.writeEntry( "notification",                 notificationDuration_         );
    profileGroup.writeEntry( "personalMessage",              personalMessage_.getOriginal());
    profileGroup.writeEntry( "saveChatPath",                 saveChatPath_                 );
    profileGroup.writeEntry( "saveChats",                    saveChats_                    );
    profileGroup.writeEntry( "savePassword",                 savePassword_                 );
    profileGroup.writeEntry( "savedChatDirectoryStructure",  savedChatDirectoryStructure_  );
    profileGroup.writeEntry( "shakeNudge",                   shakeNudge_                   );
    profileGroup.writeEntry( "showAllowed",                  showAllowedContacts_          );
    profileGroup.writeEntry( "showByGroup",                  showContactsByGroup_          );
    profileGroup.writeEntry( "showEmail",                    showEmail_                    );
    profileGroup.writeEntry( "showMessageTime",              showMessageTime_              );
    profileGroup.writeEntry( "showNowListening",             showNowListening_             );
    profileGroup.writeEntry( "showOfflineContacts",          showOfflineContacts_          );
    profileGroup.writeEntry( "showOtherFolders",             showOtherFolders_             );
    profileGroup.writeEntry( "showRemoved",                  showRemovedContacts_          );
    profileGroup.writeEntry( "showSidebar",                  showSidebar_                  );
    profileGroup.writeEntry( "initialStatus",                (int)initialStatus_           );
    profileGroup.writeEntry( "tabbedChatMode",               tabbedChatMode_               );
    profileGroup.writeEntry( "useAutologin",                 useAutologin_                 );
    profileGroup.writeEntry( "useChatFormatting",            useChatFormatting_            );
    profileGroup.writeEntry( "useContactFont",               useContactFont_               );
    profileGroup.writeEntry( "useEmoticons",                 useEmoticons_                 );
    profileGroup.writeEntry( "useFontEffects",               useFontEffects_               );
    profileGroup.writeEntry( "useListFormatting",            useListFormatting_            );
    profileGroup.writeEntry( "useIdleTimer",                 useIdleTimer_                 );
    profileGroup.writeEntry( "showPicture",                  showPicture_                  );

    if( ! savePassword_ )
    {
      // Remove the cached password
      password_ = QString();
      temporaryPassword_= QString();

      // When storing the passwords, it will be removed from the password manager also
    }
  }
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  else
  {
    kDebug() << "Account " << handle_ << " is NOT 'dirty', don't save properties.";
  }
#endif

  // Save to disk the changes now, to avoid losing data.
  profileGroup.config()->sync();
}



// The login succeeded with the new password. Save the temporary password as new account password.
void Account::saveTemporaryPassword()
{
  if( ! temporaryPassword_.isEmpty() && temporaryPassword_ != password_ )
  {
    password_ = temporaryPassword_;
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Set common alerts information
void Account::setAlertsInformation( bool useIdleTimer, int idleTime, bool shakeNudge, bool showNowListening )
{
  useIdleTimer_ = useIdleTimer;
  idleTime_ = idleTime;
  shakeNudge_ = shakeNudge;

  emit changedNotificationsSettings();
  emit changedTimerSettings();

  if( showNowListening_ != showNowListening )
  {
#ifdef KMESSDEBUG_ACCOUNT_GENERAL
    kDebug() << "Now listening settings changed, notifying listeners.";
#endif

    showNowListening_ = showNowListening;
    emit changedNowListeningSettings();
  }

  dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
}



// Set notification alerts information
void Account::setNotificationsInformation( bool doNotifyContactsOnline, bool doNotifyContactsStatus, bool doNotifyContactsChatStart,
                                           bool doNotifyContactsChatMessages, bool doNotifyContactsOffline, bool doNotifyInboxEmails, bool doNotifyOtherEmails, int notificationDuration )
{
  doNotifyContactsOnline_ = doNotifyContactsOnline;
  doNotifyContactsStatus_ = doNotifyContactsStatus;
  doNotifyContactsChatStart_ = doNotifyContactsChatStart;
  doNotifyContactsChatMessages_ = doNotifyContactsChatMessages;
  doNotifyContactsOffline_ = doNotifyContactsOffline;
  doNotifyInboxEmails_ = doNotifyInboxEmails;
  doNotifyOtherEmails_ = doNotifyOtherEmails;
  notificationDuration_ = notificationDuration;

  emit changedNotificationsSettings();

  dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
}



// Set the auto-reply message.
void Account::setAutoreplyMessage( const QString& autoreplyMessage)
{
  if ( autoreplyMessage_ != autoreplyMessage )
  {
    autoreplyMessage_ = autoreplyMessage;
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Set chat information
void Account::setChatInformation( bool useContactFont, bool useEmoticons, bool useFontEffects,
                                  bool useListFormatting, bool useChatFormatting,
                                  bool showMessageTime, bool groupFollowupMessages, int tabbedChatMode,
                                  const QString& chatStyle )
{
  bool changedSettings = (useContactFont_        != useContactFont)
                      || (useEmoticons_          != useEmoticons)
                      || (useFontEffects_        != useFontEffects)
                      || (useListFormatting_     != useListFormatting)
                      || (useChatFormatting_     != useChatFormatting)
                      || (showMessageTime_       != showMessageTime)
                      || (groupFollowupMessages_ != groupFollowupMessages)
                      || (tabbedChatMode_        != tabbedChatMode)
                      || (chatStyle_             != chatStyle);

  // Avoid signals because the chat windows will regenerate all messages.
  if( ! changedSettings )
  {
    return;
  }

  useContactFont_        = useContactFont;
  useEmoticons_          = useEmoticons;
  useFontEffects_        = useFontEffects;
  useListFormatting_     = useListFormatting;
  useChatFormatting_     = useChatFormatting;
  showMessageTime_       = showMessageTime;
  groupFollowupMessages_ = groupFollowupMessages;
  tabbedChatMode_        = tabbedChatMode;
  chatStyle_             = chatStyle;

  emit changedChatStyleSettings();
  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
}



// Set chat logging information
void Account::setChatLoggingInformation( bool saveChats, const QString& saveChatPath, const int& directoryStructure )
{
  if( saveChats_                   != saveChats
  ||  saveChatPath_                != saveChatPath
  ||  savedChatDirectoryStructure_ != directoryStructure )
  {
    saveChats_                   = saveChats;
    saveChatPath_                = saveChatPath;
    savedChatDirectoryStructure_ = directoryStructure;
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Set email information
void Account::setEmailInformation( bool showEmail, bool showOtherFolders )
{
  if( showEmail_        != showEmail
  ||  showOtherFolders_ != showOtherFolders )
  {
    showEmail_        = showEmail;
    showOtherFolders_ = showOtherFolders;

    emit changedEmailDisplaySettings();
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Update the email-supported state (protected method)
void Account::setEmailSupported(bool emailSupported)
{
  if( emailSupported_ != emailSupported )
  {
    dirty_ = true;
  }

  emailSupported_ = emailSupported; 
}



// Set the font
void Account::setFont( const QFont& font )
{
  if( font != font_ )
  {
    font_ = font;
    emit changedFontSettings();
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Set the font color
void Account::setFontColor( const QString& fontColor )
{
  if( fontColor != fontColor_ )
  {
    fontColor_ = fontColor;
    emit changedFontSettings();
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}




// Set the fonts and font colors
void Account::setFontInformation( const QFont& font, const QString &fontColor, const QFont& contactFont, const QString &contactFontColor)
{
  bool changedSettings = (font_ != font)
                      || (fontColor_ != fontColor)
                      || (contactFont_ != contactFont)
                      || (contactFontColor_ != contactFontColor);

  // Avoid signals because the chat windows will regenerate all messages.
  if( ! changedSettings )
  {
    return;
  }

  font_             = font;
  fontColor_        = fontColor;
  contactFont_      = contactFont;
  contactFontColor_ = contactFontColor;
  emit changedFontSettings();
  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
}



// Set the emoticon path
void Account::setEmoticonStyle( const QString& emoticonStyle )
{
  if( emoticonStyle != emoticonStyle_ )
  {
    // Set the new style, and signal EmoticonManager about it.
    emoticonStyle_ = emoticonStyle;
    emit changedEmoticonSettings();

    // Tell the account settings changed and need to be saved
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Change the friendly name
void Account::setFriendlyName(const QString& newName)
{
  if( newName.isEmpty() || newName == friendlyName_.getOriginal() )
  {
    return;
  }

  friendlyName_.setString( newName );

  emit changedFriendlyName();
  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  kDebug() << "Setting 'dirty' to true.";
#endif
}



// Whether the account is a guest account, not permanently saved on the computer
void Account::setGuestAccount( bool guestAccount )
{
  guestAccount_ = guestAccount;

  // Undelete when user did restore the account.
  if( ! guestAccount_ )
  {
    deleted_ = false;
  }
}



// Change the status to set upon connection
void Account::setInitialStatus( Status status )
{
  if( initialStatus_ == status )
  {
    return;
  }

  initialStatus_ = status;
  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  kDebug() << "Setting 'dirty' to true.";
#endif
}



// Set the mode of display list pictures
void Account::setListPictureViewMode( int mode )
{
  // No need to update
  if( mode == listPictureViewMode_ )
  {
    return;
  }

  listPictureViewMode_ = mode;

  emit changedDisplaySettings();
  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  kDebug() << "Setting 'dirty' to true.";
#endif
}



// Set basic account data
void Account::setLoginInformation( QString handle, QString friendlyName, QString password )
{
#ifdef KMESSTEST
  KMESS_ASSERT(   handle.contains("@") );
  KMESS_ASSERT( ! handle.isEmpty()     );
#endif

  handle = handle.toLower();

  if( handle_                          == handle
  &&  friendlyName_.getOriginal()      == friendlyName
  &&  ( password.isEmpty() || password == password_ ) )
  {
    return;
  }

  handle_ = handle;

  // Never change the nickname if the account is not verified. We would receive a server error
  if( verified_ )
  {
    friendlyName_.setString( friendlyName );
    emit changedFriendlyName();
  }

  if( ! password.isEmpty() )
  {
    password_ = password;
  }

  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  kDebug() << "Setting 'dirty' to true.";
#endif
}



// Set whether to show popup notifications when contacts come online.
void Account::setNotifyContactsOnline( bool doNotifyContactsOnline )
{
  doNotifyContactsOnline_ = doNotifyContactsOnline;
}



// Set whether to show popup notifications when contacts change their msn status.
void Account::setNotifyContactsStatus( bool doNotifyContactsStatus )
{
  doNotifyContactsStatus_ = doNotifyContactsStatus;
}



// Set whether to show popup notifications when contacts start a chat with you.
void Account::setNotifyContactsChatStart( bool doNotifyContactsChatStart )
{
  doNotifyContactsChatStart_ = doNotifyContactsChatStart;
}



// Set whether to show popup notifications for all received chat messages.
void Account::setNotifyContactsChatMessages( bool doNotifyContactsChatMessages )
{
  doNotifyContactsChatMessages_ = doNotifyContactsChatMessages;
}



// Set whether to show popup notifications when contacts go offline.
void Account::setNotifyContactsOffline( bool doNotifyContactsOffline )
{
  doNotifyContactsOffline_ = doNotifyContactsOffline;
}



// Set whether to show popup notifications for mail incoming in the Inbox.
void Account::setNotifyInboxEmails( bool doNotifyInboxEmails )
{
  doNotifyInboxEmails_ = doNotifyInboxEmails;
}



// Set whether to show popup notifications for mail incoming in other folders.
void Account::setNotifyOtherEmails( bool doNotifyOtherEmails )
{
  doNotifyOtherEmails_ = doNotifyOtherEmails;
}



// Set the account's login password
void Account::setPassword( const QString &password )
{
  password_ = password;

  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  kDebug() << "Setting 'dirty' to true.";
#endif
}



// Set the personal message
void Account::setPersonalMessage(const QString& newMessage)
{
  if( newMessage == personalMessage_.getOriginal() )
  {
    return;
  }

  personalMessage_.setString( newMessage );
  emit changedPersonalMessage();

  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  kDebug() << "Setting 'dirty' to true.";
#endif
}


// Set the display picture file
void Account::setPicturePath( const QString& picturePath )
{
  if( displayPicturePath_ == picturePath )
  {
    return;
  }

  displayPicturePath_ = picturePath;

  // Update the MSN Object linked to this account
  updateMsnObject();

  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  kDebug() << "Setting 'dirty' to true.";
#endif
}



// Set if the password must be stored
void Account::setSavePassword( bool savePassword )
{
  savePassword_ = savePassword;

  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  kDebug() << "Setting 'dirty' to true.";
#endif
}



// Set whether or not allowed contacts are shown.
void Account::setShowAllowedContacts( bool showAllowedContacts )
{
  if( showAllowedContacts != showAllowedContacts_ )
  {
    showAllowedContacts_ = showAllowedContacts;
    emit changedDisplaySettings();
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}




// Set whether contacts are shown by group or by online/offline.
void Account::setShowContactsByGroup( bool showContactsByGroup )
{
  if ( showContactsByGroup != showContactsByGroup_ )
  {
    showContactsByGroup_ = showContactsByGroup;
    emit changedViewMode();
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Set whether offline contacts should be shown.
void Account::setShowOfflineContacts( bool showOfflineContacts )
{
  if( showOfflineContacts != showOfflineContacts_ )
  {
    showOfflineContacts_ = showOfflineContacts;
    emit changedDisplaySettings();
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Set whether or not removed (reverse) contacts are shown.
void Account::setShowRemovedContacts( bool showRemovedContacts )
{
  if( showRemovedContacts != showRemovedContacts_ )
  {
    showRemovedContacts_ = showRemovedContacts;
    emit changedDisplaySettings();
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Set whether or not the sidebar should be shown in chats
void Account::setShowSidebar( bool showSidebar )
{
  if ( showSidebar_ != showSidebar )
  {
    showSidebar_ = showSidebar;
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Set the way the chats will be grouped together
void Account::setTabbedChatMode( int tabbedChatMode )
{
  tabbedChatMode_ = tabbedChatMode;
}



// Set a temporary password for the account for login attempts from the initialview dialog
void Account::setTemporaryPassword( const QString &temporaryPassword )
{
  temporaryPassword_ = temporaryPassword;
}



// Set whether or not to login with this account.
void Account::setUseAutologin( bool useAutologin)
{
  if( useAutologin != useAutologin_ )
  {
    useAutologin_ = useAutologin;
    dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  }
}



// Set whether or not to show emoticons in chats.
void Account::setUseEmoticons( bool useEmoticons)
{
  useEmoticons_ = useEmoticons;
  dirty_ = true;
#ifdef KMESSDEBUG_ACCOUNT_DIRTY
    kDebug() << "Setting 'dirty' to true.";
#endif
  emit changedChatStyleSettings();
}



// Set whether the account's passport is verified
void Account::setVerified( bool verified )
{
  verified_ = verified;
}



// Set whether or not to send our display picture
void Account::setShowPicture( bool showPicture )
{
  if( showPicture == showPicture_ )
  {
    return;
  }

  showPicture_ = showPicture;
  dirty_ = true;

#ifdef KMESSDEBUG_ACCOUNT_DIRTY
  kDebug() << "Setting 'dirty' to true.";
#endif
}



// Creates the MsnObject only if the user wants to show a display picture
void Account::updateMsnObject()
{
  QString oldMsnString;
  QString newMsnString;

  if( msnObject_ )
  {
    oldMsnString = msnObject_->objectString();
    delete msnObject_;
    msnObject_ = 0;
  }

  // Verify if the picture exists: if not, change it to the default one as returned
  // by getPicturePath()
  QString picturePath( getPicturePath() );
  if( picturePath != displayPicturePath_ )
  {
#ifdef KMESSDEBUG_ACCOUNT_GENERAL
    kDebug() << "Replacing picture" << displayPicturePath_ << "with" << picturePath;
#endif
    displayPicturePath_ = picturePath;
  }

#ifdef KMESSDEBUG_ACCOUNT_GENERAL
  kDebug() << "Show the picture?" << showPicture_ << "- Set picture path:" << displayPicturePath_;
#endif

  if ( showPicture_ )
  {
    QFile iFile( displayPicturePath_ );
    if ( iFile.open( QIODevice::ReadOnly ) )
    {
      QByteArray data = iFile.readAll();
      iFile.close();

      msnObject_   = new MsnObject( handle_, "KMess.tmp", "", MsnObject::DISPLAYPIC, data );
      newMsnString = msnObject_->objectString();
    }
    else
    {
      kWarning() << "Picture could not be loaded: " << displayPicturePath_;
    }
  }

  // Detect if there was a change.
  if(oldMsnString != newMsnString)
  {
#ifdef KMESSDEBUG_ACCOUNT_GENERAL
    kDebug() << "result is: '" << newMsnString << "'";
#endif
    emit changedMsnObject();
  }
}



#include "account.moc"
