/***************************************************************************
                          contactaction.cpp  -  description
                             -------------------
    begin                : Wed Jan 22 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "contactaction.h"

#include "../contact/contact.h"
#include "../currentaccount.h"
#include "../kmessdebug.h"

#include <KMenu>



// The constructor
ContactAction::ContactAction(const Contact *contact, KMenu *menu, QWidget *parent )
 : KAction(parent),
   contact_(contact),
   inChat_(false),
   menu_(menu),
   added_(false)
{
#ifdef KMESSTEST
  KMESS_ASSERT( menu_ != 0 );
  KMESS_ASSERT( contact_ != 0 );
#endif

  // Connect the action to its own activated signal
  connect( this,     SIGNAL(     triggered(Qt::MouseButtons,Qt::KeyboardModifiers) ),
           this,     SLOT  ( slotActivated()                                       ) );

  // Connect the contact
  connect( contact_, SIGNAL( changedFriendlyName() ),
           this,     SLOT  (          updateText() ) );
  connect( contact_, SIGNAL(       changedStatus() ),
           this,     SLOT  (    updateVisibility() ) );
  connect( contact_, SIGNAL(         changedList(Contact*) ),
           this,     SLOT  (    updateVisibility() ) );
  
  // Set default text and visibility
  updateText();
  updateVisibility();
}



// The destructor
ContactAction::~ContactAction()
{
}



// Return the contact's handle
const QString ContactAction::getHandle() const
{
  return contact_->getHandle();
}



// Set whether or not the contact is in the chat
void ContactAction::setInChat( bool inChat )
{
  inChat_ = inChat;
  updateVisibility();
}



//Inherit from KAction...
void ContactAction::slotActivated()
{
  emit activated( contact_->getHandle() );
}



// Update the text based on the contact's name
void ContactAction::updateText()
{
  setText( contact_->getFriendlyName() );
}



// Update the visibility of the action based on the contact's status, group, and list
void ContactAction::updateVisibility()
{
  bool shouldBeAdded = ( ! inChat_ && contact_->isOnline() && contact_->isFriend() );

  // Add the contact to the menu if it should be added and it's not
  if ( shouldBeAdded && ! added_ )
  {
    menu_->addAction( this );
    added_ = true;
  }
  else if ( ! shouldBeAdded && added_ )
  { // Pull the contact out of the menu if it's added but shouldn't be
    menu_->removeAction( this );
    added_ = false;
  }
}

#include "contactaction.moc"
