/***************************************************************************
                          emoticonsidebar.h -  description
                             -------------------
    begin                : Mon 12 09 2005
    copyright            : (C) 2005 by Diederik van der Boor
    email                : "vdboor" --at-- "codingdomain.com"
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef EMOTICONSIDEBAR_H
#define EMOTICONSIDEBAR_H

#include <QListWidget>


class AddEmoticonDialog;
class Emoticon;
class EmoticonListWidget;
class EmoticonManager;
class QLabel;
class QMouseEvent;
class QPoint;
class QResizeEvent;
class QVBoxLayout;



/**
 * Implements an emoticon chooser as sidebar tab.
 *
 * @author Diederik van der Boor, Valerio Pilo
 * @ingroup Chat
 */
class EmoticonSidebar : public QWidget
{
  Q_OBJECT

  public:
    // The constructor
                       EmoticonSidebar( QWidget *parent, bool useCustomTheme );
    // The destructor
    virtual           ~EmoticonSidebar();

  private slots:
    // One item was clicked
    void               slotItemClicked( const QListWidgetItem *item );
    // The "Click to open the custom emoticons settings" link has been clicked
    void               slotNoEmoticonsLinkClicked( const QString &url );
    // The theme has changed, we need to reload it
    void               slotThemeUpdated();
    // Right mouse button was clicked
    void               showContextMenu(const QListWidgetItem *item, const QPoint &point );

  private:  // Private enumerations
    enum ButtonRole
    {
      ShortcutRole = Qt::UserRole  // Role to store the button's shortcut
    , PictureRole                  // Role to store the button's picture path
    };

  private:  // private properties
    // Emoticon dialog for edit the custom emoticons
    AddEmoticonDialog *emoticonDialog_;
    // Whether the sidebar contains a standard or custom theme
    bool               isCustom_;
    // Vertical Layout which will lay our list
    QVBoxLayout       *layout_;
    // The list widget of emoticons
    EmoticonListWidget    *list_;
    // A warning message to tell the user he/she's got no custom emoticons
    QLabel            *noEmoticonsWarning_;

    // Reference to the emoticon manager, to update the emoticon buttons
    EmoticonManager   *manager_;

  signals: // Signals
    // Request that the emoticon's html code should be inserted in the message.
    void               insertEmoticon(QString html);
};



// Extend the class QListWidget to grep the mousePressEvent
// because the use of showContextMenu in main class calls also the clicked slot.
class EmoticonListWidget : public QListWidget
{
  Q_OBJECT

  public:
                  EmoticonListWidget( QWidget *parent );

  protected:
    // Reimplement the mouse press event to grep left and right click
    void          mousePressEvent( QMouseEvent *e );

  signals:
    void          leftClicked ( const QListWidgetItem* );
    void          rightClicked( const QListWidgetItem*, const QPoint& );
};


#endif
