/***************************************************************************
                          inkvedit.cpp -  description
                             -------------------
    begin                : Wed May 14 2008
    copyright            : (C) 2008 by Antonio Nastasi
    email                : sifcenter@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "inkedit.h"

#include <KDebug>
#include <KIcon>
#include <KColorDialog>

#include <QBuffer>
#include <QMouseEvent>
#include <QPainter>
#include <QPen>


 InkEdit::InkEdit(QWidget *parent)
  :  QWidget(parent),
     color_(Qt::black)
    ,isEmpty_(true)
    ,scribbling_(false)
{
  setupUi( this );
  setAttribute( Qt::WA_StaticContents );

  // Set the buttons
  colorButton_->setIcon( KIcon( "format-stroke-color" ) );
  clearButton_->setIcon( KIcon( "draw-eraser"         ) );

  connect( colorButton_, SIGNAL(    clicked()  ),
           this,         SLOT(  changeColor()  ) );
  connect( clearButton_, SIGNAL(    clicked()  ),
           this,         SLOT(   clearImage()  ) );

  // Reset the drawed area
  rect_.setTopLeft(     QPoint( -1, -1 ) );
  rect_.setBottomRight( QPoint( -1, -1 ) );
}



// Clear the current image
void InkEdit::clearImage()
{
  image_.fill( qRgb( 255, 255, 255 ) );
  update();
  isEmpty_ = true;

  // Reset the drawed area
  rect_.setTopLeft(     QPoint( -1, -1 ) );
  rect_.setBottomRight( QPoint( -1, -1 ) );

  emit inkModified( true );
}



// Change color for current pen
void InkEdit::changeColor()
{
  QColor color;
  int result = KColorDialog::getColor( color );
  if ( result == KColorDialog::Accepted )
  {
    color_ = color;
  }
}



// Crop the image to send only the drawed area
void InkEdit::cropImage( QPoint position )
{
  int sizePen = sizePen_->value();

  // Adjust the initial size and position
  if( rect_.topLeft().x() == - 1 || rect_.topLeft().y() == -1 )
  {
    rect_.setTopLeft( QPoint( position.x(), position.y() ) );
    rect_.setBottomRight( QPoint( position.x() + sizePen , position.y() + sizePen ) );
    return;
  }

  // Adjust the x position and width
  if( position.x() < rect_.topLeft().x() )
  {
    rect_.setTopLeft( QPoint( position.x() - sizePen , rect_.topLeft().y() ) );
  }
  else if( position.x() > rect_.bottomRight().x() )
  {
    rect_.setBottomRight( QPoint( position.x() + sizePen, rect_.bottomRight().y() ) );
  }

  // Adjust the y position and height
  if( position.y() < rect_.topLeft().y() )
  {
    rect_.setTopLeft( QPoint( rect_.topLeft().x(), position.y( )  - sizePen ) );
  }
  else if( position.y() > rect_.bottomRight().y() )
  {
    rect_.setBottomRight( QPoint( rect_.bottomRight().x() , position.y()+ sizePen ) );
  }
}



// Draw a line from start point to last point
void InkEdit::drawLineTo( const QPoint &endPoint )
{
  QPainter painter( &image_ );
  painter.setPen( QPen( color_, sizePen_->value(), Qt::SolidLine, Qt::RoundCap,
                  Qt::RoundJoin ) );

  painter.drawLine( lastPoint_, endPoint );

  int rad = (1 / 2) + 2;
  update( QRect( lastPoint_, endPoint ).normalized()
          .adjusted( -rad, -rad, +rad, +rad ) );

  lastPoint_ = endPoint;
}



// Return if the image is empty
bool InkEdit::isEmpty()
{
  return isEmpty_;
}



void InkEdit::mousePressEvent( QMouseEvent *event )
{
  if( event->button() == Qt::LeftButton )
  {
    lastPoint_ = event->pos();
    cropImage( lastPoint_ );
    scribbling_ = true;
  }
}



void InkEdit::mouseMoveEvent(QMouseEvent *event)
{
  if( ( event->buttons() & Qt::LeftButton ) && scribbling_ )
  {
    QPoint position = event->pos();
    cropImage(  position );
    drawLineTo( position );
  }
}



void InkEdit::mouseReleaseEvent(QMouseEvent *event)
{
  if( event->button() == Qt::LeftButton && scribbling_ )
  {
    QPoint position = event->pos();
    cropImage(  position );
    drawLineTo( position );

    isEmpty_ = false;
    scribbling_ = false;
    emit inkModified();
  }
}



void InkEdit::paintEvent(QPaintEvent */*event*/)
{
  QPainter painter( this) ;
  painter.drawImage( QPoint(0, 0), image_ );
}



void InkEdit::resizeEvent( QResizeEvent *event )
{
  /*int newWidth = width();
  int newHeight = height();
  kDebug() << image_.size();
  kDebug() << newWidth;
  resizeImage( &image_, QSize( newWidth, newHeight ) );
  update();*/
  if( width() > image_.width() || height() > image_.height() ) 
  {
    int newWidth = qMax( width() + 128, image_.width() );
    int newHeight = qMax( height() + 128, image_.height() );
    resizeImage( &image_, QSize( newWidth, newHeight ) );
    update();
  }

  QWidget::resizeEvent( event );
}



// Return the bytes rapresents the image
const QByteArray& InkEdit::getImageBytes()
{
  QBuffer buffer( &imageBytes_ );
  buffer.open( QIODevice::WriteOnly );
  image_.copy( rect_ ).save( &buffer, "PNG" );
 //image_.scaled( size(), Qt::KeepAspectRatio, Qt::SmoothTransformation ).save( &buffer, "PNG" );
  return imageBytes_;
}



void InkEdit::resizeImage( QImage *image, const QSize &newSize )
{
  if( image->size() == newSize )
  {

    return;
  }

  QImage newImage( newSize, QImage::Format_ARGB32 );
  newImage.fill( qRgb( 255, 255, 255 ) );
  QPainter painter( &newImage );
  if( ! image->isNull() )
  {
    painter.drawImage( QPoint( 0, 0 ), (*image).scaled( newSize, Qt::KeepAspectRatio, Qt::SmoothTransformation ) );
  }
  else
  {
    painter.drawImage( QPoint( 0, 0 ), *image );
  }

  *image = newImage;
}

#include "inkedit.moc"
