/***************************************************************************
                          inkedit.cpp -  description
                             -------------------
    begin                : Wed May 14 2008
    copyright            : (C) 2008 by Antonio Nastasi
    email                : sifcenter@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INKEDIT_H
#define INKEDIT_H

#include "ui_inkedit.h"

#include <QWidget>

class InkEdit : public QWidget, public Ui::InkEdit
{
  Q_OBJECT

  public:
                        InkEdit(QWidget *parent = 0);
    // Return the bytes rapresents the image
    const QByteArray    &getImageBytes();
    // Return if the image is empty
    bool                isEmpty();

  public slots:
    // Clear the current image
    void                clearImage();

  protected: // protected methods
    // Implements all events to grep the mouse pointer
    void                mousePressEvent( QMouseEvent *event );
    void                mouseMoveEvent( QMouseEvent *event );
    void                mouseReleaseEvent( QMouseEvent *event );
    void                paintEvent( QPaintEvent *event );
    void                resizeEvent( QResizeEvent *event );

  private: // private methods
    // Crop the image to send only the drawed area
    void                cropImage( QPoint position );
    // Draw a line from start point to last point
    void                drawLineTo( const QPoint &endPoint );
    // Resize image method
    void                resizeImage( QImage *image, const QSize &newSize );

  private slots:
    // Change color for current pen
    void                changeColor();

  private: // private attribute
    // Color of current pen
    QColor              color_;
    // Variable to store if the image is empty or not
    bool                isEmpty_;
    // The bytes of image
    QByteArray          imageBytes_;
    // The main image
    QImage              image_;
    // Last point where the mouse pointer was released
    QPoint              lastPoint_;
    // Rect to store the position and size of current drawed area
    QRect               rect_;
    // Return if we are in writing or no
    bool                scribbling_;

  signals:
    // Emitted when the ink was modified
    void                inkModified( bool erasered = false );
};

#endif
