/***************************************************************************
                          contactextension.h  -  description
                             -------------------
    begin                : Thu 12 Dec 2002
    copyright            : (C) 2003 by Michael Curtis
    email                : mdcurtis@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTACTEXTENSION_H
#define CONTACTEXTENSION_H

#include "../utils/richtextparser.h"
#include "contactbase.h"
#include "contact.h"

#include <QObject>


// forward declarations
class Contact;
class KConfigGroup;



/**
 * @brief Additional KMess-specific properties of a contact.
 *
 * The class stores "extended" info for a contact, such as an alternative friendlyName.
 * This information is not received from the MSN Messenger servers,
 * the Contact class is reserved for that information.
 * Most of these properties are set by the user from the ContactPropertiesDialog.
 *
 * @author Michael Curtis
 * @ingroup Contact
 */
class ContactExtension : public QObject
{
  Q_OBJECT

  public:
    // The constructor with specified contact handle
    	                     ContactExtension( Contact *contact );
    // The destructor
    	                    ~ContactExtension();

    // Clear cache
    void                   clearCache();
    // Read property of QString alternativeName_.
    const QString &        getAlternativeName( FormattingMode mode = STRING_CLEANED ) const;
    // Read property of QString alternativePictureFile_
    const QString &        getContactAlternativePicturePath() const;
    // Read property of QString pictureFile_.
    const QString          getContactPicturePath() const;
    // Read property of QString soundFile_.
    const QString          getContactSoundPath() const;
    // Read property of QString lastSeen_
    const QString&         getLastSeen();
    // Read property of QString notes_
    const QString&         getNotes() const;
    // Read property of bool notifyOnline_
    bool                   getNotifyOnline() const;
    // Read property of bool useAlternativeName_.
    bool                   getUseAlternativeName() const;
    // Read property of pictures list
    QStringList            getPictureList() const;
     // Load extension settings for a contact
    void                   readProperties( const KConfigGroup &config );
    // Save extension settings for a contact
    void                   saveProperties( KConfigGroup &config );
    // Write property of QString alternativeName_.
    void                   setAlternativeName( const QString& newAlternativeName );
    // Write property of QString alternativePictureFile_
    void                   setContactAlternativePicturePath( const QString& _newVal );
    // Write property of QString pictureFile_.
    void                   setContactPicturePath( const QString& _newVal );
    // Write property of QString soundFile_.
    void                   setContactSoundPath( const QString& _newVal );
    // Write property of QString notes
    void                   setNotes( const QString& _newVal );
    // Write property of bool notifyOnline_
    void                   setNotifyOnline( bool _newVal );
    // Write property of bool useAlternativeName_;
    void                   setUseAlternativeName( bool _newVal );


  private: // Private attributes
    // The alternative name
    FormattedString        alternativeName_;
    // The alternative picture
    QString                alternativePictureFile_;
    // The current contact
    Contact               *contact_;
    // The contact's handle (ie email)
    QString                handle_;
    // The LastSeen string
    QString                lastSeen_;
    // The contact's notes string
    QString                notes_;
    // Whether or not to notify online/offline status
    bool                   notifyOnline_;
    // A picture used for the "contact is online" balloons
    QString                pictureFile_;
    // A list of all pictures used by the contact
    QStringList            pictureFileList_;
    // A sound played when the contact is online
    QString                soundFile_;
    // Whether or not we should use the alternative name stored in the class
    bool                   useAlternativeName_;

  signals: // Signals
    // Notifiy any watchers that this contact's information has been changed.
    void                   changedFriendlyName();
    // Signal that the contact's picture path has changed
    void                   changedPicture();

  private slots:
    // Write property of QString LastSeen
    void                   setLastSeen();

};
#endif
