/***************************************************************************
                          contactstatus.h
                             -------------------
    begin                : Saturday March 8th 2008
    copyright            : (C) 2008 by Richard Conway
    email                : richardconway1984 hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MSNSTATUS_H
#define MSNSTATUS_H

#include <QPixmap>



/**
 * Available MSN states
 * @ingroup Contact
 */
enum Status
{
    STATUS_ONLINE      = 0
  , STATUS_AWAY
  , STATUS_BE_RIGHT_BACK
  , STATUS_BUSY
  , STATUS_INVISIBLE
  , STATUS_IDLE
  , STATUS_OFFLINE
  , STATUS_ON_THE_PHONE
  , STATUS_OUT_TO_LUNCH
};


/**
 * Contact flags
 *
 * These can be used to apply icon overlays to the status icons
 * @ingroup Contact
 */
enum Flags
{
    FlagNone         = 0
  , FlagBlocked
  , FlagWebMessenger
  , FlagMobile
};


/**
 * Contact Status management class
 *
 * This class contains everything we need to manage the MSN states.
 * The main enumeration which contains the MSN states can be used
 * along the conversion methods, which translate the enum values to
 * string (the three letter character codes like NLN for Online) and
 * to localized strings (like "Out to lunch"). There's also a method
 * to translate the status values to the relative status icon.
 *
 * @author Valerio Pilo <valerio@kmess.org>
 * @author Richard Conway <richardconway1984 at hotmail.com>
 * @ingroup Contact
 */
class MsnStatus
{
  public: // Public static methods

    // Returns an icon for a status
    static QPixmap  getIcon( const Status status, const Flags flags = FlagNone );
    // Returns a three-letter code (like BSY) for a status
    static QString  getCode( const Status status );
    // Returns the localized name for a status
    static QString  getName( const Status status );
    // Returns the localized name for a status
    static QString  getStatus( const Status status );
    // Convert a three-letter code to a status
    static Status   codeToStatus( const QString &status );
};


#endif
