/***************************************************************************
                          contactlistviewdelegate.h -  description
                             -------------------
    begin                : Sat Feb 16 2008
    copyright            : (C) 2008 by Valerio Pilo
    email                : valerio@kmess.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTACTLISTVIEWDELEGATE_H
#define CONTACTLISTVIEWDELEGATE_H

#include <QAbstractItemDelegate>


// Forward declarations
class CurrentAccount;
class KIconLoader;
class QTextDocument;


/**
 * Displays a contact list item using its model
 *
 * This class is used by a Contact List view to display a single contact
 * or group. It receives the data to display from the Contact List model and paints
 * it directly.
 *
 * @author Valerio Pilo <valerio@kmess.org>
 * @ingroup Root
 */
class ContactListViewDelegate : public QAbstractItemDelegate
{
  public:
    // Constructor
                 ContactListViewDelegate( QWidget *parent = 0 );
    // Destructor
                ~ContactListViewDelegate();

    // Paint an item of the contact list
    void         paint( QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index ) const;
    // Return an hint about the size of an item
    QSize        sizeHint( const QStyleOptionViewItem &option, const QModelIndex &index ) const;

  private:  // Private properties
    // Reference to the current account's properties
    CurrentAccount *currentAccount_;
    // Icon loader to make paint a bit faster
    KIconLoader *iconLoader_;
    // Text Document for rich text
    QTextDocument *textDocument_;
};

#endif
