/***************************************************************************
                          contactpropertiesdialog.h  -  description
                             -------------------
    begin                : Sun Dec 15 2002
    copyright            : (C) 2002 by Michael Curtis
    email                : magnalium@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTACTPROPERTIESDIALOG_H
#define CONTACTPROPERTIESDIALOG_H

#include "ui_contactpropertiesdialog.h"

#include <KDialog>
#include <Phonon/MediaObject>


class Contact;



/**
 * The dialog for modifying contact properties (ContactExtension).
 *
 * @author Michael Curtis
 * @ingroup Dialogs
 */
class ContactPropertiesDialog : public KDialog, private Ui::ContactProperties
{
   Q_OBJECT

  public:
                 ContactPropertiesDialog( QWidget *parent=0 );
    virtual      ~ContactPropertiesDialog();
    // Show the dialog and obtain the contact handle.
    bool         launch( Contact *contact, bool openNotesTab = false );

  protected:
    bool         eventFilter(QObject *obj, QEvent *event);

  private: // Private methods
    void         applyChanges();
    void         setupWidgets();
    // Get the location of a file from the user, using a KFile dialog
    void         chooseFile(KLineEdit *edit, QString wildcard);

  private slots:
    // The alt name checkbox was toggled.
    void         alternativeNameCheckBoxToggled(bool checked);
    // Choose a picture file (KFile dialog)
    void         choosePicture();
    // Choose a sound file (KFile dialog)
    void         chooseSound();
    // The online notification checkbox was toggled
    void         notifyOnlineCheckBoxToggled(bool checked);
    // Restore the original contact picture
    void         restoreContactPicture();
    // The Ok or Cancel button has been pressed
    void         slotButtonClicked( int button );
    // The "Clear cache" button has been pressed
    void         slotClearCache();
    // The play sound button has been pressed
    void         slotPlaySound();
    // Called when media sound finished
    void         slotSoundFinished();

  private: // Private attributes
    // Alternative picture path
    QString             alternativePicturePath_;
    // Contact
    Contact             *contact_;
    // Whether or not OK was pressed.
    bool                ok_;
    // Sound media object
    Phonon::MediaObject *mediaSound_;

  signals:
    void                addContactToGroup     ( QString contact, QString id );
    void                removeContactFromGroup( QString contact, QString id );
};

#endif
