/***************************************************************************
                          userpicturesdialog.cpp  -  description
                             -------------------
    begin                : Fri Jul 7 2008
    copyright            : (C) 2008 by Antonio Nastasi
    email                : sifcenter@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "userpicturesdialog.h"

#include "../utils/kmessconfig.h"
#include "../kmessdebug.h"

#include <QDir>


UserPicturesDialog::UserPicturesDialog( QWidget *parent )
  :  QWidget( parent )
{
  setupUi(this);
  setWindowTitle( i18n( "Choose or remove your avatar" ) );

  useButton_    ->setText( i18n( "Use"    ) );
  deleteButton_ ->setText( i18n( "Remove" ) );

  connect( listPictures_, SIGNAL(     itemSelectionChanged() ),
           this,            SLOT( slotItemSelectionChanged() ) );

  connect( useButton_,    SIGNAL( clicked() ), this, SLOT(    useButtonClicked() ) );
  connect( deleteButton_, SIGNAL( clicked() ), this, SLOT( deleteButtonClicked() ) );
}



UserPicturesDialog::~UserPicturesDialog()
{
  kDebug() << "DESTROYED";
}



// Delete button was clicked
void UserPicturesDialog::deleteButtonClicked()
{
  // Check if the current item is valid
  QListWidgetItem *currentItem = listPictures_->currentItem();
  if( currentItem == 0 )
  {
    return;
  }

  // Check if the file exists and remove it
  QString filename = currentItem->data( 6 ).toString();
  if( QFile::exists( filename ) )
  {
    QFile::remove( filename );
  }

  // Update the list
  updateWidgets( handle_ );
}



// Current item was changed
void UserPicturesDialog::slotItemSelectionChanged()
{
  QListWidgetItem *currentItem = listPictures_->currentItem();

  if( currentItem == 0 )
  {
    useButton_   ->setEnabled( false );
    deleteButton_->setEnabled( false );
  }
  else
  {
    // Set current item to avoid lost the blue rect on selection
    listPictures_->setCurrentItem( currentItem );
    useButton_   ->setEnabled( true );
    deleteButton_->setEnabled( true);
  }
}



// Use button was clicked
void UserPicturesDialog::useButtonClicked()
{
  // Check if the current item is valid
  QListWidgetItem *currentItem = listPictures_->currentItem();
  if( currentItem == 0 )
  {
    return;
  }

  emit pictureUpdate( false, currentItem->data( 6 ).toString() );
  hide();
}



// Update the widgets
void UserPicturesDialog::updateWidgets( const QString &handle )
{
  int number = 0;
  QString filePath;
  QString dirPath = KMessConfig::instance()->getAccountDirectory( handle );
  QDir dir( dirPath + "/displaypics" );
  QListWidgetItem *item;

  handle_ = handle;
  listPictures_->clear();

  foreach( QString file, dir.entryList( QDir::Files | QDir::NoDotAndDotDot ) )
  {
    // To avoid listing of temporary picture file
    if( file.contains( "temporary-picture" ) )
    {
      continue;
    }

    filePath = dir.absolutePath() + "/" + file;
    item = new QListWidgetItem( QIcon( filePath ), QString::number( ++number ), listPictures_ );
    item->setData( 6, filePath );
  }

  useButton_   ->setEnabled( false );
  deleteButton_->setEnabled( false );
}

#include "userpicturesdialog.moc"
