/***************************************************************************
                          initialview.h  -  description
                             -------------------
    begin                : Sun Jan 12 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INITIALVIEW_H
#define INITIALVIEW_H

#include "ui_initialview.h"
#include "contact/msnstatus.h"

#include <QHash>
#include <QWidget>


// Forward declarations
class Account;



/**
 * @brief The login widget.
 *
 * Users can login by selecting an existing account, or entering their username/password directly.
 * When the login starts, the AutoLoginView widget is displayed in the main window.
 * This class implements the events defined in the inherited user interface class.
 *
 * @author Mike K. Bennett
 * @ingroup Root
 */
class InitialView : public QWidget, private Ui::InitialView
{
  Q_OBJECT

  public:
    // The constructor
                         InitialView( QWidget *parent );
    // The destructor
    virtual             ~InitialView();
    // Add an account to the list of displayed accounts from which the user can choose
    void                 addAccount( Account *account );
    // Modify an account in the list of displayed accounts from which the user can choose
    void                 changedAccount( QString oldName, QString newName );
    // The account was deleted
    void                 deleteAccount(Account *account);
    // Start the connections: disable all widgets and set the handle if there is autologin
    void                 startConnecting( const QString handle = QString() );

  public slots: // Public slots
    // A profile was selected from the drop-down list, or written manually.
    void                 updateView();
    // Change the connection status text
    void                 statusMessage( QString text );

  private slots:
    // Connect to MSN with the entered account data
    void                 connectToMsn();
    // The "connect" button was pressed, schedule connection
    void                 connectToMsnDelayed();
    // Get the currently selected handle
    QString              getSelectedHandle() const;
    // The "remember account" checkbox change its state
    void                 rememberAccountStateChanged( int state );
    // Toggle displaying the Options box, which allows remembering accounts & passwords
    void                 toggleOptionsBox();

  private: // Private attributes
    // The list of accounts to choose from
    QHash<QString,Account*>       accounts_;
    // The application's configuration file
    KConfigGroup         config_;
    // The default handle to display
    QString              defaultHandle_;
    // Loader for icons
    KIconLoader          *loader_;

  signals: // Public signals
    // Connect with the given account
    void                 connectWithAccount( QString handle, QString password, bool rememberAccount, bool rememberPassword, Status initialStatus );
    // Disconnect button was clicked
    void                 disconnectClicked();
};

#endif
