/***************************************************************************
                          kmessinterface.cpp  -  description
                             -------------------
    begin                : Sun Dec 29 2002
    copyright            : (C) 2002 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmessinterface.h"

#include "settings/globalsettingsdialog.h"
#include "utils/kmessconfig.h"
#include "utils/kmessshared.h"
#include "kmessapplication.h"
#include "kmessdebug.h"
#include "systemtraywidget.h"

#include <QHBoxLayout>
#include <QLabel>
#include <QPoint>
#include <QSize>
#include <QStringList>

#include <KAction>
#include <KActionMenu>
#include <KConfig>
#include <KGlobal>
#include <KHelpMenu>
#include <KLocale>
#include <KMenu>
#include <KMenuBar>
#include <KMessageBox>
#include <KRun>
#include <KSelectAction>
#include <KStandardAction>
#include <KStatusBar>
#include <KToggleAction>
#include <KToolBar>
#include <KToolInvocation>
#include <KUrl>



// The constructor
KMessInterface::KMessInterface( QWidget *parent )
 : KMainWindow( parent )
 , isErrorStatus_(true)
 , initialized_(false)
{
}



// The destructor
KMessInterface::~KMessInterface()
{
  delete systemTrayWidget_;
#ifdef KMESSDEBUG_KMESSINTERFACE
  kDebug() << "DESTROYED.";
#endif
}


// Show the "about KDE" message.
void KMessInterface::aboutKDE()
{
  helpMenu_->aboutKDE();
}



// Show the "about KMess" message.
void KMessInterface::aboutKMess()
{
  helpMenu_->aboutApplication();
}



// "Add a new contact" was selected from the menu.
void KMessInterface::addNewContact()
{
  kDebug() << "not implemented";
}



// "Add a new group" was selected from the menu.
void KMessInterface::addNewGroup()
{
  kDebug() << "not implemented";
}


// The application is closing, after queryExit() was approved
void KMessInterface::applicationClosing()
{
  kDebug() << "not implemented";
}



// A view pictures mode has been selected from the menu.
void KMessInterface::changedListPictureViewMode( int /*mode*/ )
{
  kDebug() << "not implemented";
}



// A status was selected from the menu.
void KMessInterface::changeStatus( int /*index*/ )
{
  kDebug() << "not implemented";
}



// A view mode has been selected from the menu.
void KMessInterface::changeViewMode(int /*mode*/)
{
  kDebug() << "not implemented";
}


void KMessInterface::showNetworkWindow()
{
  kDebug() << "not implemented";
}


void KMessInterface::showTransferWindow()
{
  kDebug() << "not implemented";
}



// Create the "Actions" menu
void KMessInterface::createActionsMenu()
{
  KAction       *newAccount, *searchForContact, *searchByInterest;
  KToggleAction *searchInContactList;
  KMenu         *actionsMenu;

  // Create the menu
  actionsMenu = new KMenu( i18n("&Actions"), this );

  // Create the "settings" menus
  newContact_         = new KAction( KIcon("list-add-user"),  i18n("New &Contact"),        this );
  newGroup_           = new KAction( KIcon("user-group-new"), i18n("New &Group"),          this );
  searchForContact    = new KAction( KIcon("edit-find-user"), i18n("Search for Contact"),  this );
  searchByInterest    = new KAction( KIcon("system-search"),  i18n("Search by &Interest"), this );

  searchInContactList = new KToggleAction( KIcon("edit-find-user"), i18n("Search in Contact List"),  actionCollection_ );
  searchInContactList->setShortcut( QKeySequence::fromString( tr( "Ctrl+f" ) ) );
  actionCollection_->addAction("edit-find-user", searchInContactList);

  newAccount          = new KAction( KIcon("user-identity"),  i18n("New &Account"),        this );
  globalSettings_     = new KAction( KIcon("kmess"),          i18n("Configure &KMess"),    this );

  connect( newContact_,      SIGNAL(                  triggered(bool) ),
                               SLOT(              addNewContact()     ) );
  connect( newGroup_,        SIGNAL(                  triggered(bool) ),
                               SLOT(                addNewGroup()     ) );
  connect( searchForContact, SIGNAL(                  triggered(bool) ),
                               SLOT(           searchForContact()     ) );
  connect( searchByInterest, SIGNAL(                  triggered(bool) ),
                               SLOT( searchForContactByInterest()     ) );
  connect( searchInContactList, SIGNAL(               triggered(bool) ),
                              SLOT(       toggleShowSearchFrame(bool) ) );
  connect( newAccount,       SIGNAL(                  triggered(bool) ),
                               SLOT(           createNewAccount()     ) );
  connect( globalSettings_,  SIGNAL(                  triggered(bool) ),
                               SLOT(         showGlobalSettings()     ) );

  // preferences-desktop-user
  settingsActionMenu_     = new KActionMenu( KIcon("configure"), i18n("Account &Settings"), this );

  // Plug the items into "settings"
  actionsMenu->addAction( newContact_ );
  actionsMenu->addAction( newGroup_ );
  actionsMenu->addSeparator();
  actionsMenu->addAction( searchForContact );
  actionsMenu->addAction( searchByInterest );
  actionsMenu->addAction( searchInContactList );
  actionsMenu->addSeparator();
  actionsMenu->addAction( newAccount );
  actionsMenu->addAction( settingsActionMenu_ );
  actionsMenu->addAction( globalSettings_ );

  // Plug the menu into the menubar
  menuBar()->addMenu( actionsMenu );
}



// Create the "Connect" menu
void KMessInterface::createConnectMenu()
{
  KAction     *close, *quit;
  KMenu       *connectMenu;
  QStringList  states;

  // Create the menu
  connectMenu  = new KMenu( i18n("&Connect"), this );

  // Create the "connect" menu items
  connectActionMenu_ = new KActionMenu(   KIcon("network-connect"), i18n("Co&nnect"), this );
  disconnect_        = new KAction(       KIcon("network-disconnect"), i18n("&Disconnect"), this );
  showProfile_       = new KAction(       KIcon("preferences-desktop-user"), i18n("Show My &Profile"), this );
  status_            = new KSelectAction( KIcon("go-jump"), i18n("&My Status"), this );


  states << i18n("Online")
         << i18n("Away")
         << i18n("Away with auto-reply")
         << i18n("Idle")
         << i18n("Be Right Back")
         << i18n("Busy")
         << i18n("Out to Lunch")
         << i18n("On the Phone")
         << i18n("Invisible");
  status_->setItems(states);

  connect( disconnect_,  SIGNAL(         triggered(bool) ),
           this,           SLOT( disconnectClicked()     ) );
  connect( showProfile_, SIGNAL(         triggered(bool) ),
           this,           SLOT(   showUserProfile()     ) );
  connect( status_,      SIGNAL(         triggered(int)  ),
           this,           SLOT(      changeStatus(int)  ) );
 
  close = KStandardAction::close( this, SLOT( menuClose() ), actionCollection_ );
  quit  = KStandardAction::quit ( this, SLOT(  menuQuit() ), actionCollection_ );

  // Plug the items into "connect"
  connectMenu->addAction( connectActionMenu_ );
  connectMenu->addAction( disconnect_ );
  connectMenu->addSeparator( );
  connectMenu->addAction( status_ );
  connectMenu->addAction( showProfile_ );
  connectMenu->addSeparator();
  connectMenu->addAction( close );
  connectMenu->addAction( quit );

 /* toolBar( "mainToolBar" )->addAction( disconnect_ );
  toolBar( "mainToolBar" )->addAction( status_ );*/

  // Plug the menu into the menubar
  menuBar()->addMenu( connectMenu );
}



// Create the "Help" menu
void KMessInterface::createHelpMenu()
{
  KAction       *aboutKDE, *aboutKMess, *helpContents;
  KMenu    *helpMenu;

  // Create the menu
  helpMenu     = new KMenu( i18n("&Help"), this );

  // Create the help menu.
  helpMenu_ = new KHelpMenu(this, KGlobal::mainComponent().aboutData());

  // Create the help menu items
  helpContents = KStandardAction::helpContents( this, SLOT( helpContents() ), actionCollection_ );
  aboutKMess   = KStandardAction::aboutApp    ( this, SLOT(   aboutKMess() ), this );
  aboutKDE     = KStandardAction::aboutKDE    ( this, SLOT(     aboutKDE() ), this );

  // Plug the items into "help-contents"
  helpMenu->addAction( helpContents );
  helpMenu->addSeparator();
  helpMenu->addAction( aboutKMess );
  helpMenu->addAction( aboutKDE );

  // Plug the menu into the menubar
  menuBar()->addMenu( helpMenu );
}



// Create the menus
void KMessInterface::createMenus()
{
  // ActionCollection to collect the actions with a shortcut
  actionCollection_ = new KActionCollection( this, KComponentData() );

  // Create the 4 menus
  createConnectMenu();
  createViewMenu();
  createActionsMenu();
  createHelpMenu();

  // Add collected actions
  addActions( actionCollection_->actions() );
}



// Create the "View" menu
void KMessInterface::createViewMenu()
{
  QStringList      viewModes, showPictureModes;

  // Create the menu
  viewMenu_ = new KMenu( i18n("&View"), this );

  // Create the "view" menu items
  showAllowedAction_ = new KToggleAction( i18n("Show &Allowed Contacts"), this );
  showOfflineAction_ = new KToggleAction( i18n("Show &Offline Contacts"), this );
  showRemovedAction_ = new KToggleAction( i18n("Show &Removed Contacts"), this );

  // The list mode display picture
  listPictureViewMode_ = new KSelectAction( KIcon("view-list-tree"), i18n("Display Pictures Size"), this );
  showPictureModes  << i18n("Do not display") 
                    << i18n("Smallest")
                    << i18n("Small"   )
                    << i18n("Medium"  )
                    << i18n("Large"   )
                    << i18n("Largest" );
  listPictureViewMode_->setItems( showPictureModes );

  viewMode_ = new KSelectAction( KIcon("view-list-tree"), i18n("&Sort Contacts by"), this );
  viewModes << i18n("Group") << i18n("Online/Offline");
  viewMode_->setItems(viewModes);

  showTransferAction_ = new KAction( KIcon("document-open-remote"), i18n("Show &Transfer Window"), this );

  //showToolBar_   = new KToggleAction( i18n("Show &Tool bar"), this );
  showStatusBar_ = KStandardAction::showStatusbar( this, SLOT( showStatusBar() ), this );
  showMenuBar_   = KStandardAction::showMenubar(   this, SLOT(   showMenuBar() ), actionCollection_ );

#ifdef KMESS_NETWORK_WINDOW
  showNetworkAction_ = new KAction( KIcon("network-workgroup"), i18n("Show &Network Window"), this );
  connect( showNetworkAction_,  SIGNAL( triggered(bool) ), this, SLOT( showNetworkWindow() ) );
#endif

  // Connect the actions
//  connect( showToolBar_,  SIGNAL( triggered(bool) ), this, SLOT(           showToolBar()     ) );
  connect( viewMode_,           SIGNAL( triggered(int)  ), this, SLOT(             changeViewMode(int) ) );
  connect( listPictureViewMode_,SIGNAL( triggered(int)  ), this, SLOT( changedListPictureViewMode(int) ) );
  connect( showAllowedAction_,  SIGNAL( triggered(bool) ), this, SLOT(         toggleShowAllowed(bool) ) );
  connect( showOfflineAction_,  SIGNAL( triggered(bool) ), this, SLOT(         toggleShowOffline(bool) ) );
  connect( showRemovedAction_,  SIGNAL( triggered(bool) ), this, SLOT(         toggleShowRemoved(bool) ) );
  connect( showTransferAction_, SIGNAL( triggered(bool) ), this, SLOT(            showTransferWindow() ) );

  // Plug the items into "view"
  viewMenu_->addAction( showAllowedAction_  );
  viewMenu_->addAction( showRemovedAction_  );
  viewMenu_->addAction( showOfflineAction_  );
  viewMenu_->addSeparator();
  viewMenu_->addAction( listPictureViewMode_);
  viewMenu_->addSeparator();
  viewMenu_->addAction( viewMode_           );
  viewMenu_->addSeparator();
  viewMenu_->addAction( showTransferAction_ );
  viewMenu_->addSeparator();
//  viewMenu_->addAction( showToolBar_        );
  viewMenu_->addAction( showMenuBar_        );
  viewMenu_->addAction( showStatusBar_      );

#ifdef KMESS_NETWORK_WINDOW
  viewMenu_->addSeparator();
  viewMenu_->addAction( showNetworkAction_  );
#endif

  // Plug the menu into the menubar
  menuBar()->addMenu( viewMenu_ );
}



// "Add new account" has been selected from the menu.
void KMessInterface::createNewAccount()
{
  kDebug() << "not implemented";
}



// Disconnect was selected from the menu.
void KMessInterface::disconnectClicked()
{
  kDebug() << "not implemented";
}



// Enable/disable menus based on whether or not the application is
//  connected to the server.
void KMessInterface::enableMenus(bool connected)
{
  connectActionMenu_->setEnabled( !connected );
  disconnect_->setEnabled( connected );
  if ( connected )
  {
    disconnect_->setIcon( KIcon("network-connect") );
  }
  else
  {
    disconnect_->setIcon( KIcon("network-disconnect") );
  }
  status_->setEnabled( connected );

  newContact_         ->setEnabled( connected );
  newGroup_           ->setEnabled( connected );
  showAllowedAction_  ->setEnabled( connected );
  showOfflineAction_  ->setEnabled( connected );
  showRemovedAction_  ->setEnabled( connected );
  listPictureViewMode_->setEnabled( connected );
  showProfile_        ->setEnabled( connected );
  viewMode_           ->setEnabled( connected );
  // showNetworkAction should ALWAYS be enabled!
}



// Show the user manual for KMess.
void KMessInterface::helpContents()
{
  KToolInvocation::invokeHelp("kmess/index");
}



// Initialize the class
bool KMessInterface::initialize()
{
#ifdef KMESSDEBUG_KMESSINTERFACE
  kDebug() << "initializing";
#endif
  if ( initialized_ )
  {
    kDebug() << "already initialized.";
    return false;
  }

  // Initialize the status bar
  statusLabel_ = new KSqueezedTextLabel( this );
  statusBar()->addWidget( statusLabel_, 2 );

  statusTimer_ = new QLabel( this );
  statusTimer_->setText( "00:00" );
  statusBar()->addPermanentWidget( statusTimer_, 0 );

  // Only show the status bar when we're connected
  statusBar()->hide();

  // Create the online timer
  onlineTimer_ = new QTimer( this );
  connect( onlineTimer_,      SIGNAL( timeout()           ),
           this,                SLOT( updateOnlineTimer() ) );

  // Create the menus
  createMenus();

  if ( ! initSystemTrayWidget() )
  {
    kDebug() << "Couldn't initialize the system tray widget.";
    return false;
  }

#ifdef KMESSDEBUG_KMESSINTERFACE
  kDebug() << "initialized";
#endif
  return true;
}



// Initialize the system tray widget
bool KMessInterface::initSystemTrayWidget()
{
  bool initialized;

  // Create the widget
  systemTrayWidget_ = new SystemTrayWidget( this ) ;
  if ( systemTrayWidget_ == 0 )
  {
    kDebug() << "Couldn't create system tray widget.";
    return false;
  }

  // Initialize the widget
  initialized = systemTrayWidget_->initialize();
  if ( !initialized )
  {
    kDebug() << "Couldn't initialize system tray widget.";
    return false;
  }

  // Plug some of the menu actions into the system tray widget's menu.
  QMenu *menu = systemTrayWidget_->menu();
  menu->addAction( connectActionMenu_ );
  menu->addAction( disconnect_ );
  menu->addSeparator();
  menu->addAction( status_ );
  menu->addSeparator();
  menu->addAction( settingsActionMenu_ );
  menu->addAction( globalSettings_ );

  // Make the connections for the system tray widget
  connect( systemTrayWidget_, SIGNAL( quitSelected() ),
           this,              SLOT  (     menuQuit() ) );

  systemTrayWidget_->show();

#ifdef KMESSTEST
  KMESS_ASSERT( systemTrayWidget_ != 0 );
  KMESS_ASSERT( systemTrayWidget_->isVisible() );
#endif
  return true;
}



// Close has been selected from the menu.
void KMessInterface::menuClose()
{
#ifdef KMESSDEBUG_KMESSINTERFACE
  kDebug() << "'Close' was selected from the menu.";
#endif

  // Hide the window
  // Tell that KMess is still visible in the tray
  hide();
  systemTrayWidget_->displayCloseMessage();

#ifdef KMESSTEST
  KMESS_ASSERT( isHidden() );
#endif
}



// Quit was selected from the menu.
void KMessInterface::menuQuit()
{
#ifdef KMESSDEBUG_KMESSINTERFACE
  kDebug() << "'Quit' was selected from the menu.";
#endif

  // Tell the application manager we want to quit
  KMessApplication *app = static_cast<KMessApplication *>(kapp);
  app->setQuitSelected(true);

  close();  // Close this window, initiates quit
}



// Reject quitting unless the quit menu was pressed  Called automatically by KMainWindow::closeEvent
bool KMessInterface::queryExit()
{
  KMessApplication *kmessApp = static_cast<KMessApplication *>(kapp);

  // With KDE 4 this function is also called when another KMainWindow (like the network window) closes.
  // Reject an attempt to quit if the user didn't select quit manually, or is logging out from KDE.
  // If a window still causes KMess to quit, use setAttribute( Qt::WA_QuitOnClose, false ) on it.
  if( kmessApp->quitSelected() ||
      kmessApp->sessionSaving() )
  {
#ifdef KMESSDEBUG_KMESSINTERFACE
    kDebug() << "Accepting exit request.";
#endif

    return true;
  }
  else
  {
    // For some reason, KDE keeps bugging us when the chat window closes.
    // Reject the request again.

#ifdef KMESSDEBUG_KMESSINTERFACE
    kDebug() << "Rejecting exit request! Keep running with all windows hidden.";
#endif

    return false;
  }
}



// Tell the user that KMess hides in the systray  Called automatically by KMainWindow::closeEvent
bool KMessInterface::queryClose()
{
  KMessApplication *kmessApp = static_cast<KMessApplication *>(kapp);

  // Only allow KMess to quit if:
  // - the "quit" option was used from the menu
  // - the session manager wants to quit (we're called from KApplication::commitData())
  if( kmessApp->quitSelected() ||
      kmessApp->sessionSaving() )
  {
#ifdef KMESSDEBUG_KMESSINTERFACE
    kDebug() << "Accepting quit request.";
#endif

    // Make sure that substituent also return true
    kmessApp->setQuitSelected(true);

    // Make sure the derived class can prepare itself too.
    applicationClosing();

    // Allow the request
    return true;
  }
  else
  {
    // Tell that KMess is still visible in the tray 
    systemTrayWidget_->displayCloseMessage();

#ifdef KMESSDEBUG_KMESSINTERFACE
    kDebug() << "Rejecting quit request, hiding window";
#endif

    // Only hide, disallow event
    hide();
    return false;
  }
}



// Restore the window properties (called by KMainWindow)
void KMessInterface::readProperties( const KConfigGroup &config )
{
#ifdef KMESSDEBUG_KMESSINTERFACE
  kDebug() << "Reading properties";
#endif

  KConfigGroup group;

  // Choose the config group if it was not given
  if( &config == 0 )
  {
    group = KMessConfig::instance()->getGlobalConfig( "ContactListWindow" );
  }
  else
  {
    group = config;
  }

  // Pull in the window size and position
  QSize windowsize = group.readEntry( "Size", QSize() );
  if(!windowsize.isEmpty())
  {
    resize(windowsize);
  }
  QPoint position = group.readEntry( "Position", QPoint() );
  if(!position.isNull())
  {
    move(position);
  }

  bool bViewMenubar = group.readEntry( "Show Menubar", true );
  showMenuBar_->setChecked( bViewMenubar );
  menuBar()->setVisible( bViewMenubar );

  // Commented for future work
/*  // bar status settings
  bool bViewToolbar = group.readEntry( "Show Toolbar", true );
  showToolBar_->setChecked(bViewToolbar);
  showToolBar();
*/

  bool bViewStatusbar = group.readEntry( "Show Statusbar", true );
  showStatusBar_->setChecked( bViewStatusbar );
  showStatusBar();

  // bar position settings
 /* KConfigGroup mainToolBarGroup = group.group( "mainToolBar" );
  toolBar( "mainToolBar" )->applySettings( mainToolBarGroup );
  toolBar( "mainToolBar" )->setToolBarsLocked( group.readEntry( "Lock Toolbar", false ) );*/
}



// Save the window properties (called by KMainWindow)
void KMessInterface::saveProperties(KConfigGroup &config)
{
#ifdef KMESSDEBUG_KMESSINTERFACE
  kDebug() << "Saving properties";
#endif

  KConfigGroup group;

  // Choose the config group if it was not given
  if( &config == 0 )
  {
    group = KMessConfig::instance()->getGlobalConfig( "ContactListWindow" );
  }
  else
  {
    group = config;
  }

  group.writeEntry("Size",           size()                      );
  group.writeEntry("Position",       pos()                       );
//  group.writeEntry("Show Toolbar",   showToolBar_->isChecked()   );
  group.writeEntry("Show Menubar", showMenuBar_->isChecked());
  group.writeEntry("Show Statusbar", showStatusBar_->isChecked() );

  // save the toolbar settings
 /* KConfigGroup mainToolBarGroup = group.group( "mainToolBar" );
  toolBar( "mainToolBar" )->saveSettings( mainToolBarGroup );
  group.writeEntry("Lock Toolbar", toolBar( "mainToolBar" )->toolBarsLocked());*/

#ifdef KMESSDEBUG_KMESSINTERFACE
  kDebug() << "Done saving properties";
#endif
}



// Open up the URL for MSN's contact search page.
void KMessInterface::searchForContact()
{
  // Launch the browser for the given URL
  KMessShared::openBrowser( KUrl( "http://members.msn.com/find.msnw?mkt=" + KGlobal::locale()->country() ) );
}



// Open up the URL to MSN's "search by interest" page.
void KMessInterface::searchForContactByInterest()
{
  // Launch the browser for the given URL
  KMessShared::openBrowser( KUrl( "http://members.msn.com/rootcat.msnw?mkt=" + KGlobal::locale()->country() ) );
}



// The "Show Global Settings" option was clicked.
void KMessInterface::showGlobalSettings()
{
  GlobalSettingsDialog *globalSettingsDialog = GlobalSettingsDialog::instance();

  globalSettingsDialog->show();
  globalSettingsDialog->raise();
}



// "Show menu bar" was toggled.
void KMessInterface::showMenuBar()
{
  if( ! showMenuBar_->isChecked() )
  {
    menuBar()->hide();
  }
  else
  {
    menuBar()->show();
  }
}



// "Show status bar" was toggled.
void KMessInterface::showStatusBar()
{
  if(!showStatusBar_->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }
}



// "Show toolbar" was toggled.
/*void KMessInterface::showToolBar()
{
  if(!showToolBar_->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }
}*/



// Show the user's MSN profile.
void KMessInterface::showUserProfile()
{
  kDebug() << "not implemented";
}



// Change the status bar message.
void KMessInterface::statusMessage( QString message, bool isError )
{
  if( message != message_ )
  {
    statusLabel_->setText( message );
    message_ = message;

#ifdef KMESSDEBUG_KMESSINTERFACE
    kDebug() << "Changed message to" << message;
#endif
  }

  // Do not alter the online timer when the status has not changed
  if( isError == isErrorStatus_ )
  {
    return;
  }

  if( isError )
  {
      onlineTimer_->stop();
      statusTimer_->setText( "00:00" );
  }
  else
  {
      onlineTime_ = 0;
      onlineTimer_->start( 60000 );
  }

  isErrorStatus_ = isError;
}



// The "show allowed contacts" menu item has been toggled.
void KMessInterface::toggleShowAllowed(bool /*show*/)
{
  kDebug() << "not implemented";
}



// The "show search in contact list" menu item has been toggled.
void KMessInterface::toggleShowSearchFrame(bool /*show*/)
{
  kDebug() << "not implemented";
}



// The "show offline contacts" menu item has been toggled.
void KMessInterface::toggleShowOffline(bool /*show*/)
{
  kDebug() << "not implemented";
}



// The "show removed contacts" menu item has been toggled.
void KMessInterface::toggleShowRemoved(bool /*show*/)
{
  kDebug() << "not implemented";
}


// Increment and update the online timer.
void KMessInterface::updateOnlineTimer()
{
  QString timeText;

  onlineTime_ ++;

  timeText.sprintf( "%02i:%02i", onlineTime_ / 60, onlineTime_ % 60 );

  statusTimer_->setText( timeText );
}

#include "kmessinterface.moc"
