/***************************************************************************
                          kmesstest.cpp  -  description
                             -------------------
    begin                : Sun Jan 5 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmesstest.h"

#include "chat/chat.h"
#include "chat/chatmaster.h"
#include "chat/chatview.h"
#include "chat/chatwindow.h"
#include "chat/contactsidebar.h"
#include "chat/xsltransformation.h"
#include "contact/contact.h"
#include "contact/contactbase.h"
#include "contact/contactextension.h"
#include "contact/contactlist.h"
#include "contact/group.h"
#include "dialogs/addcontactdialog.h"
#include "dialogs/contactaddeduserdialog.h"
#include "dialogs/transferentry.h"
#include "dialogs/transferwindow.h"
#include "network/soap/httpsoapconnection.h"
#include "network/upnp/manager.h"
#include "network/soap/msnappdirectoryservice.h"
#include "network/soap/soapmessage.h"
#include "network/msnnotificationconnection.h"
#include "network/msnswitchboardconnection.h"
#include "notification/contactstatusnotification.h"
#include "notification/passivepopup.h"
#include "settings/accountsettingsdialog.h"
#include "utils/nowlisteningclient.h"
#include "utils/richtextparser.h"
#include "contactlistviewdelegate.h"
#include "currentaccount.h"
#include "emoticonmanager.h"
#include "kmess.h"
#include "kmessapplication.h"
#include "kmessview.h"
#include "systemtraywidget.h"

#include <QFile>
#include <QImage>
#include <QRegExp>
#include <QTreeView>

#include <KMessageBox>
#include <KStandardDirs>



// The constructor
KMessTest::KMessTest(KMess *kmess)
 : QObject()
 , kmess_(kmess)
{
  setObjectName("KMessTest");
}



// The destructor
KMessTest::~KMessTest()
{
  kDebug() << "DESTROYED";
}


#ifdef KMESSTEST

#define  TESTCASE(name, action) if( testName == name ) { action; return; }


/**
 * The main testing function.
 * Runs a given test based on a command line argumet.
 */
void KMessTest::runTest( const QString &testName )
{
  // Add the tests, which can be run in the debug build using:
  // ./kmess --runtest <name>
  kDebug() << "Running test " << testName << "\n\n\n"
              "----------------------------------------";

  TESTCASE( "transferwindow",   testTransferWindow() );
  TESTCASE( "chatwindow",       testChatWindow() );
  TESTCASE( "crash",            testCrash() );
  TESTCASE( "addcontact",       testAddContactDialog() );
  TESTCASE( "contactaddeduser", testContactAddedUserDialog() );
  TESTCASE( "msnplus",          testMsnPlus() );
  TESTCASE( "ns",               testNotificationConnection() );
  TESTCASE( "contactlist",      testContactList() );
  TESTCASE( "connect",          testConnect() );
  TESTCASE( "settings",         testSettings() );
  TESTCASE( "sb",               testSwitchboardConnection() );
  TESTCASE( "notifications",    testNotifications() );
  TESTCASE( "nowlistening",     testNowListening() );
  TESTCASE( "soap",             testSoapConnection() );
  TESTCASE( "upnp",             testUpnpConnection() );
  TESTCASE( "xslt",             testXslTransformation() );
  TESTCASE( "systemtray",       testSystemTray() );

  kDebug() << endl << "----------------------------------------";
  kDebug() << "Use Ctrl+C to quit";
}



void KMessTest::testTransferWindow()
{
  int id;
  KIconLoader *loader = KIconLoader::global();
  TransferWindow *instance = TransferWindow::getInstance();

  id = instance->addEntry( "Filename1", 1234567890, true, QImage( loader->iconPath( "list-add", 48, false ) ) );
  kDebug() << "Added ID: " << id;

  id = instance->addEntry( "Filename2", 34567890, false, QImage( loader->iconPath( "list-remove", 48, false ) ) );
  kDebug() << "Added ID: " << id;

  id = instance->addEntry( "Filename3", 567890, true );
  kDebug() << "Added ID: " << id;

//   TransferWindow::destroy();
}


void KMessTest::testChatWindow()
{
  testGroupsAndContacts();

  EmoticonManager::instance()->connected();
  CurrentAccount::instance()->setTabbedChatMode( 0 );
  CurrentAccount::instance()->setFontColor( "#ff9900" );

  MsnSwitchboardConnection *sb1 = new MsnSwitchboardConnection();
  ChatInformation chatInfo( kmess_->msnNotificationConnection_, "contact1@kmess.org", -1, ChatInformation::CONNECTION_OFFLINE );
  sb1->start( chatInfo );
  sb1->initialize( "contact1@kmess.org" );

  MsnSwitchboardConnection *sb2 = new MsnSwitchboardConnection();
  sb2->initialize( "contact2@kmess.org" );
  sb2->contactJoined( "contact2@kmess.org", "Contact 2", 0 );
  sb2->contactJoined( "contact5@kmess.org", "Contact 5", 0 );

  MsnSwitchboardConnection *sb3 = new MsnSwitchboardConnection();
  sb3->initialize( "contact7@kmess.org" );
  sb3->contactJoined( "contact7@kmess.org", "Contact 7", 0 );

  kmess_->chatMaster_->createChat( sb1 );
  kmess_->chatMaster_->createChat( sb2 );
  kmess_->chatMaster_->createChat( sb3 );

//   QTimer::singleShot( 1000, this, SLOT( testSlot() ) );
}


// A generic method to use as throwaway slot for delayed code
void KMessTest::testSlot()
{
}


void KMessTest::testCrash()
{
  QObject *object = 0;
  object->objectName();
}


void KMessTest::testAddContactDialog()
{
  QString handle, groupId;
  new AddContactDialog( handle, groupId );

  kDebug() << "adding " << handle << "on group id" << groupId;
}


void KMessTest::testContactAddedUserDialog()
{
  ContactAddedUserDialog *dialog = new ContactAddedUserDialog();
  ContactAddedUserDialog::ReturnCode code = dialog->launch( "someone@hotmail.com", "SomeNickName" );

  kDebug() << "result is " << code;
}


void KMessTest::testNotifications()
{
  testGroupsAndContacts();

  ContactList *list = kmess_->msnNotificationConnection_->contactList_;
  Contact *contact1 = list->getContactByHandle( "contact1@kmess.org" );
  Contact *contact2 = list->getContactByHandle( "contact2@kmess.org" );
  Contact *contact3 = list->getContactByHandle( "contact3@kmess.org" );

  contact1->setStatus( STATUS_ONLINE  );
  contact2->setStatus( STATUS_OFFLINE );
  contact3->setStatus( STATUS_BUSY    );

  ContactStatusNotification *cs = new ContactStatusNotification( kmess_->notificationManager_ );
  cs->notify( contact1, true );
  cs->notify( contact2, true );
  cs->notify( contact3, true );
}


void KMessTest::testNowListening()
{
  NowListeningClient *client = new NowListeningClient();
  client->setEnabled( true );
}


void KMessTest::testNotificationConnection()                { }
void KMessTest::testConnect()                               { }
void KMessTest::testSettings()                              { }
void KMessTest::testSwitchboardConnection()                 { }



void KMessTest::testGroupsAndContacts()
{
  ContactList *list = kmess_->msnNotificationConnection_->contactList_;

  list->reset( true );

  /*Group *group1 = */list->addGroup( "friends", "My Friends" );
  /*Group *group2 = */list->addGroup( "work", "Work Mates" );
                      list->addGroup( "empty", "Empty Group");
  Contact *contact1 = list->addContact( "contact1@kmess.org", "Contact 1 :)", 3, QStringList()<<"friends"<<"work", "c1" );
  Contact *contact2 = list->addContact( "contact2@kmess.org", "Contact 2", 3, QStringList()<<"friends", "c2" );
  Contact *contact3 = list->addContact( "contact3@kmess.org", "Contact 3", 3, QStringList()<<"work", "c3" );
  Contact *contact4 = list->addContact( "contact4@kmess.org", "Contact 4", 3, QStringList()<<"friends", "c4" );
  Contact *contact5 = list->addContact( "contact5@kmess.org", "Contact 5", 3, QStringList()<<"friends", "c5" );
  Contact *contact6 = list->addContact( "contact6@kmess.org", "Contact 6 www.google.it", 3, QStringList()<<"work"<<"friends", "c6" );
  Contact *contact7 = list->addContact( "contact7@kmess.org", "<b> Contact 7 </b>", 3, QStringList()<<"friends", "c7" );
  Contact *contact8 = list->addContact( "contact8@kmess.org", "Contact 8", 3, QStringList(), "c7" );
  contact1->setPersonalStatus( "Contact Status" );
  contact2->setPersonalStatus( "Contact Status" );
  contact3->setPersonalStatus( "Contact Status" );
  contact4->setPersonalStatus( "Contact Status", "Music", "Chemical Brothers - Got Glint?" );
  contact5->setPersonalStatus( "Contact Status" );
  contact6->setPersonalStatus( "Contact Status" );
  contact7->setPersonalStatus( "Contact Status" );
  contact8->setPersonalStatus( "Contact Status" );
  contact5->setStatus( STATUS_ONLINE, false );
}



void KMessTest::testContactList()
{
  testGroupsAndContacts();

  kmess_->enableMenus( true );
  kmess_->switchViewToContactList();
  kmess_->window()->show();

  // Disable all extra fields
  CurrentAccount::instance()->setChatInformation( true, true, true, true, true, true, true, true, "Fresh" );
  CurrentAccount::instance()->setEmailSupported( false );
  kmess_->view_->slotUpdateEmailDisplay();

  // Show the Now Listening field
//   kmess_->view_->changedSong( "White Stripes, The", "Elephant", "Seven Nation Army", true );
}



void KMessTest::testMsnPlus()
{
  QStringList strings;

  strings << QString::fromUtf8( "[c=#0080FF][u][c=39][b](S) [c=29] Fede  [/c](S)[/b][/c=39] forza azzurriiiiiii[/u][/c]" );
  strings << QString::fromUtf8( "[c=50][a=50]Opera[/a][/c][c=0][a=0]tore ec[/a][/c][c=4][a=4]ologico[/a][/c]" );
  strings << QString::fromUtf8( "[c=4]t[a=#0000]est[/c=44][/a]" );
  strings << QString::fromUtf8( "[c=3]te[a=2]st:)[/a=12]er[/c=4]" );
  strings << QString::fromUtf8( "[a=red]test[a=green]:DeroWithCo[/a=blue]lorssadfasd dfg sdfsdaf sdf [/a=yellow]");
  strings << QString::fromUtf8( "[c=#660000],.-~*'¨¯¨'*·~-.¸-).[i]Simo[/i].(-,.-~*'¨¯¨'*·~-.¸[/c=#00FFF]" );
  strings << QString::fromUtf8( "[b][a=1][c=4]- Emet -:) [/c][/a][/b]" );
  strings << QString::fromUtf8( "[b][a=1][c=10]- :) [T]ester <b>boldExploit</b> :P -[/c][/a][/b]" );
  strings << QString::fromUtf8( "[b][c=green]Vale - New kid on the block! :D[/c=blue][/b]" );
 // strings << QString::fromUtf8( "�$#F37AF0Vanitosa �$ �$#0A79F5Da strega cattiva a principessa..." );
  foreach( QString str, strings )
  {
    QString original = str;
    QString cleaned = str;
    RichTextParser::getCleanString( cleaned );
    RichTextParser::parseMsnString( str, true, true, true, true );
    RichTextParser::getFormattedString( str );
    KMessageBox::information( kmess_, "<html>" + original + "<br><br>" + cleaned  + "<br><br>" + str  + "</html>" );
  }

  exit(0);
}



void KMessTest::testSoapConnection()
{
  MsnAppDirectoryService *appDir = new MsnAppDirectoryService();
  appDir->queryServiceList( MsnAppDirectoryService::GAMES );
}



void KMessTest::testSystemTray()
{
  kmess_->systemTrayWidget_->displayCloseMessage();
}



void KMessTest::testUpnpConnection()
{
  UPnP::Manager *manager = UPnP::Manager::instance();

  Q_UNUSED(manager);
}


void KMessTest::testXslTransformation()
{
  // A simple test using XSL/XML data of the KMess website
  QString wwwRoot = "/srv/www/vhosts/kmess/htdocs";

  // Load XSL
  XslTransformation *xsl = new XslTransformation();
  xsl->setStylesheet( wwwRoot + "/resources/xsl/kmess.xsl" );

  // Open XML
  QFile file( wwwRoot + "/index.xml" );
  bool open = file.open(QIODevice::ReadOnly);
  if( ! open)
  {
    kWarning() << "Could not open file: " << file.fileName();
    return;
  }

  // Load XML
  QByteArray fileData = file.readAll();
  QString xml = QString::fromUtf8(fileData.data(), fileData.size());
  file.close();

  // Transform XML
  kDebug() << "XSL transformation result:";
  kDebug() << xsl->convertXmlString(xml);

  delete xsl;
}



#endif  // KMESSTEST

#include "kmesstest.moc"
