/***************************************************************************
                          kmesstest.h  -  description
                             -------------------
    begin                : Sun Jan 5 2003
    copyright            : (C) 2003 by Mike K. Bennett
    email                : mkb137b@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMESSTEST_H
#define KMESSTEST_H

#include "kmessdebug.h"  // for KMESSTEST

#include <QObject>


// forward declarations
class Chat;
class KMess;
class ChatWindow;



/**
 * @brief A class responsible for testing the other classes.
 *
 * This class can be used by uncommenting a few lines in main.cpp.
 * New functions can be added to implement various test cases,
 * like opening chat windows without logging in.
 * Most of these functions are included in an <code>\#ifdef</code> block,
 * so they are not compiled in the release version.
 *
 * @author Mike K. Bennett
 * @ingroup Debug
 */
class KMessTest : public QObject
{
  // No signals/slots yet, but keep the option open
  Q_OBJECT

  public:
    // The constructor
               KMessTest(KMess *kmess);
    // The destructor
    virtual   ~KMessTest();
    // The main testing function
    void       runTest( const QString &testname );

#ifdef KMESSTEST
  private: // Private methods
    void       testTransferWindow();
    void       testChatWindow();
    void       testCrash();
    void       testAddContactDialog();
    void       testContactAddedUserDialog();
    void       testMsnPlus();
    void       testNotificationConnection();
    void       testContactList();
    void       testConnect();
    void       testGroupsAndContacts();
    void       testSettings();
    void       testSwitchboardConnection();
    void       testNotifications();
    void       testNowListening();
    void       testSoapConnection();
    void       testSystemTray();
    void       testUpnpConnection();
    void       testXslTransformation();
  private slots:
    void       testSlot();
#endif

  private: // Private attributes
    // A pointer to the main widget, also used to gain access to all its attributes
    KMess     *kmess_;
};

#endif
