/***************************************************************************
                          newemailnotification.cpp - notifies when a
                            new email has arrived
                             -------------------
    begin                : Tuesday April 10 2007
    copyright            : (C) 2007 by Valerio Pilo
    email                : valerio@kmess.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "newemailnotification.h"

#include "../currentaccount.h"
#include "../kmessdebug.h"
#include "notificationmanager.h"
#include "passivepopup.h"

#include <QFile>
#include <QPixmap>
#include <QTextDocument>

#include <KLocale>
#include <KNotification>



// Class constructor
NewEmailNotification::NewEmailNotification( NotificationManager* manager )
  : doNotifyInbox_(true)
  , doNotifyOther_(false)
  , manager_(manager)
{
}



// Notify the user about this event (a new email has arrived)
void NewEmailNotification::notify( QString sender, QString subject, bool inInbox, QString command, QString folder, QString url )
{
  Q_UNUSED( url );
  Q_UNUSED( command );

  currentAccount_ = CurrentAccount::instance();

  if( inInbox && ! doNotifyInbox_ )
  {
#ifdef KMESSDEBUG_NEWEMAILNOTIFICATION
    kDebug() << "New email notification disabled for inbox messages.";
#endif
    return;
  }
  else if( ! inInbox && ! doNotifyOther_ )
  {
#ifdef KMESSDEBUG_NEWEMAILNOTIFICATION
    kDebug() << "New email notification disabled for non-inbox messages.";
#endif
    return;
  }

#ifdef KMESSDEBUG_NEWEMAILNOTIFICATION
  kDebug() << "New email from " << sender << " in inbox: " << inInbox << ".";
#endif


  QString      text;
  QStringList  stringList;
  // Command and url are not used at the moment (jun 2008)
//   stringList.append( command );
  stringList.append( folder  );
//   stringList.append( url     );

  text = i18n( "<html><b>New email:</b><br>'%1'<br>by '%2'</html>",
               Qt::escape( subject ),
               Qt::escape( sender  )
             );


  // Display a popup for this event
  PassivePopup::PopupProperties props;
  props.source = sender;
  props.message = text;
  props.icon = "envelope";
  props.status = STATUS_ONLINE;
  props.destMessages = stringList;
  props.destObject = 0;

  manager_->notify( this, props );


  // Also play a sound
#ifdef KMESSDEBUG_NEWEMAILNOTIFICATION
  kDebug() << "Play the default sound.";
#endif
  KNotification::event( "new email", text );
}



// Trigger this event's action (Open the mailbox where the new mail is)
void NewEmailNotification::activate( QStringList stringList, QObject */*object*/ )
{
  // Open hotmail at the given email
  currentAccount_->openMail( stringList[0] );
}



// Enable or disable the new email notifications
void NewEmailNotification::setEnabled( bool doNotifyInbox, bool doNotifyOther )
{
  doNotifyInbox_ = doNotifyInbox;
  doNotifyOther_ = doNotifyOther;
}



#include "newemailnotification.moc"
