/***************************************************************************
                          passivepopup.cpp - KMess' Notification Popup
                             -------------------
    begin                : Tuesday April 10 2007
    copyright            : (C) 2007 by Valerio Pilo
    email                : valerio@kmess.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "passivepopup.h"

#include "../kmessdebug.h"
#include "passivepopupview.h"
#include "passivepopupwidget.h"



// Class constructor
PassivePopup::PassivePopup( QWidget *parent, PopupProperties props )
  : properties_(props)
{
  popup_ = new PassivePopupWidget( parent );
  viewWidget_ = new PassivePopupView( popup_, properties_.icon, properties_.message, properties_.status );
  // Connect the closeRequested signal to rightClicked to hide this popup
  connect( viewWidget_, SIGNAL( closeRequested() ), this, SLOT( rightClicked() ) );

  popup_->setTimeout( properties_.timeout );
  popup_->setView( viewWidget_ );

  connect( popup_, SIGNAL( leftClicked()  ),
           this,     SLOT( leftClicked()  ) );
  connect( popup_, SIGNAL( rightClicked() ),
           this,     SLOT( rightClicked() ) );
  connect( popup_, SIGNAL( destroyed(QObject*) ),
           this,     SLOT( forwardDestroyed() ) );

}



// Change the popup's properties
bool PassivePopup::change( PopupProperties newProps )
{
  if( properties_.source != newProps.source )
  {
#ifdef KMESSDEBUG_PASSIVEPOPUP
    kDebug() << "Tried to update a popup for " << newProps.source << ", but i'm made for " << properties_.source << "!";
#endif
    return false;
  }

  if( properties_.message != newProps.message )
  {
    properties_.message = newProps.message;
    viewWidget_->changeMessage( newProps.message );
  }

  if( properties_.icon != newProps.icon )
  {
    properties_.icon = newProps.icon;
    viewWidget_->changeIcon( newProps.icon );
  }

  if( properties_.status != newProps.status )
  {
    properties_.status = newProps.status;
    viewWidget_->changeStatusIcon( newProps.status );
  }

  // NOTE: timeout is not changed..
  // seeing the update in a quick flash is better.

  return true;
}



// Obtain the current popup position
QPoint PassivePopup::getPosition()
{
  return popup_->pos();
}



// Display the popup
void PassivePopup::show()
{
  popup_->show( properties_.position );
}



// Called when it's being destroyed, to remove the popup from the stack
void PassivePopup::forwardDestroyed()
{
#ifdef KMESSDEBUG_PASSIVEPOPUP
  kDebug() << "PassivePopup:forwardDestroyed() - killing popup for " << properties_.source;
#endif

  // Disconnect all signals which go from or to this object
  emit popupDestroyed( properties_.source );
}



// Called when the popup gets left-clicked with the mouse; emits the activate() signal
void PassivePopup::leftClicked()
{
#ifdef KMESSDEBUG_PASSIVEPOPUP
  kDebug() << "Activating popup for " << properties_.source;
#endif

  emit activate( properties_.destMessages, properties_.destObject );

  popup_->hide();
}



// Called when the popup gets right-clicked with the mouse; does no actions
void PassivePopup::rightClicked()
{
#ifdef KMESSDEBUG_PASSIVEPOPUP
  kDebug() << "Hiding popup for " << properties_.source;
#endif

  popup_->hide();
}


#include "passivepopup.moc"
