/***************************************************************************
                          passivepopupview.cpp -  Displays the passive popup
                            contents
                             -------------------
    begin                : Tuesday April 10 2007
    copyright            : (C) 2007 by Valerio Pilo
    email                : valerio@kmess.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "passivepopupview.h"

#include "contact/msnstatus.h"

#include <QLabel>
#include <QPixmap>



// Class constructor
PassivePopupView::PassivePopupView( QWidget *parent, QString icon, QString message, Status status )
  : QWidget( parent )
  , Ui::PassivePopupView()
{
  // First, setup the interface widgets
  setupUi( this );

  iconLoader_ = KIconLoader::global();

  changeIcon( icon );
  changeStatusIcon( status );
  message_->setText( message );

  // Add a side image, the KMess Hummingbird, to make the popups identifiable
  // as belonging to KMess.
  kMessLogo_->setPixmap( iconLoader_->loadIcon( "kmess", KIconLoader::Desktop, KIconLoader::SizeHuge ) );
  closeButton_->setIcon( iconLoader_->loadIcon( "dialog-close", KIconLoader::Desktop, KIconLoader::SizeHuge ) );
  connect( closeButton_, SIGNAL( pressed() ), this, SIGNAL( closeRequested() ) );
}



// Changes the popup icon
void PassivePopupView::changeIcon( QString icon )
{
  QPixmap picture;

  // If the specified icon is not valid, we'll use the KMess icon
  if( ! picture.load( icon ) )
  {
    picture = iconLoader_->loadIcon( "kmesspic", KIconLoader::User );
  }

  displayPicture_->setPixmap( picture );
}



// Changes the popup status icon
void PassivePopupView::changeStatusIcon( Status status )
{
  statusPixmap_->setPixmap( MsnStatus::getIcon( status ) );
}



// Changes the popup message
void PassivePopupView::changeMessage( QString message )
{
  message_->setText( message );
}



#include "passivepopupview.moc"
