/***************************************************************************
                          emoticonspage.cpp -  description
                             -------------------
    begin                : Sat May 3 2008
    copyright            : (C) 2008 by Valerio Pilo
    email                : valerio@kmess.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "accountsmanagerpage.h"

#include "../utils/kmessconfig.h"
#include "../account.h"
#include "../accountsmanager.h"
#include "../kmessdebug.h"

#include <QDir>
#include <QListWidget>
#include <QPixmap>
#include <QPushButton>

#include <KLocale>
#include <KMessageBox>
#include <KStandardDirs>



/**
 * Constructor
 *
 * @param parent  Parent widget
 */
AccountsManagerPage::AccountsManagerPage( QWidget *parent )
: QWidget(parent)
, Ui::AccountsManagerPage()
{
  // First setup the user interface
  setupUi( this );

  // Connect the signals
  connect( addAccountButton_,       SIGNAL(              clicked()                 ),
           this,                      SLOT( showAddAccountDialog()                 ) );
  connect( configureAccountButton_, SIGNAL(              clicked()                 ),
           this,                      SLOT(     configureAccount()                 ) );
  connect( removeAccountButton_,    SIGNAL(              clicked()                 ),
           this,                      SLOT(        deleteAccount()                 ) );
  connect( accountsList_,           SIGNAL(        itemActivated(QListWidgetItem*) ),
           this,                      SLOT(     configureAccount()                 ) );
  connect( accountsList_,           SIGNAL( itemSelectionChanged()                 ),
           this,                      SLOT(      accountSelected()                 ) );

  accountsManager_ = AccountsManager::instance();

  connect( accountsManager_,        SIGNAL( accountChanged(Account*,QString,QString) ),
           this,                    SLOT  ( readProperties()                         ) );

  readProperties();
}



/**
 * Destructor
 */
AccountsManagerPage::~AccountsManagerPage()
{
}



/**
 * Update the buttons status when accounts get selected
 */
void AccountsManagerPage::accountSelected()
{
  bool hasSelection = ( ! accountsList_->selectedItems().isEmpty() );

  configureAccountButton_->setEnabled( hasSelection );
  removeAccountButton_   ->setEnabled( hasSelection );
}



/**
 * Open the currently selected account's settings
 */
void AccountsManagerPage::configureAccount()
{
  if( accountsList_->selectedItems().isEmpty() )
  {
    return;
  }

  QListWidgetItem * selection = accountsList_->selectedItems().first();
  QString accountHandle = selection->data( Qt::UserRole ).toString();

#ifdef KMESSDEBUG_SETTINGSDIALOG
  kDebug() << "Opening Account Settings for account" << accountHandle << ".";
#endif

  Account *account = accountsManager_->getAccountByHandle( accountHandle );
  accountsManager_->showAccountSettings( account, window() );
}



/**
 * Delete the currently selected account
 */
void AccountsManagerPage::deleteAccount()
{
  if( accountsList_->selectedItems().isEmpty() )
  {
    kWarning() << "No account was selected for deletion!";
    return;
  }

  QListWidgetItem *selection = accountsList_->selectedItems().first();

  Account *account = accountsManager_->getAccountByHandle( selection->data( Qt::UserRole ).toString() );
  if( ! account )
  {
    return;
  }

  int result = KMessageBox::warningYesNo(
                                 this,
                                 i18n("Are you sure you want to delete the account '%1' ?<br>"
                                      "All settings of this account will be lost.",
                                      account->getHandle() ) );
  if( result != KMessageBox::Yes )
  {
    return;
  }

  accountsManager_->deleteAccount( account );
  readProperties();
}



/**
 * Load the settings of the dialog
 */
void AccountsManagerPage::readProperties()
{
  accountsList_->clear();
  const QList<Account*> accountsList = AccountsManager::instance()->getAccounts();
  foreach( Account *account, accountsList )
  {
    QString text( account->getFriendlyName() + "\n" + account->getHandle() );
    QListWidgetItem *item = new QListWidgetItem( QIcon( account->getPicturePath() ), text, accountsList_ );
    item->setData( Qt::UserRole, account->getHandle() );
  }

  // Update the viewport and the buttons
  accountSelected();
}



/**
 * Show the Account Settings dialog for a new account
 */
void AccountsManagerPage::showAddAccountDialog()
{
  accountsManager_->showAccountSettings();
}



#include "accountsmanagerpage.moc"
