/***************************************************************************
                          alertspage.cpp -  description
                             -------------------
    begin                : Thu Jan 30 2008
    copyright            : (C) 2008 by Diederik van der Boor
    email                : "vdboor" --at-- "codingdomain.com"
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "alertspage.h"

#include "../account.h"

#include <KDebug>



/**
 * The constructor
 */
AlertsPage::AlertsPage( QWidget *parent )
: QWidget( parent )
, Ui::AlertsPage()
{
  setupUi( this );

  // Connect our signals
  connect( showChatStartNotificationsCheckBox_,  SIGNAL(              toggled(bool) ),
           this,                                 SLOT  ( notificationsToggled()     ) );
}



/**
 * @brief Load the widget state from an account
 *
 * Reads an Account's info and sets our internal widgets accordingly
 *
 * @param account The account where to pick settings from
 */
void AlertsPage::loadSettings( const Account *account )
{
  // Set the notifications widgets
  showOnlineNotificationsCheckBox_      ->setChecked( account->getNotifyContactsOnline()       );
  showChatStartNotificationsCheckBox_   ->setChecked( account->getNotifyContactsChatStart()    );
  showChatMessagesNotificationsCheckBox_->setChecked( account->getNotifyContactsChatMessages() );
  showStatusNotificationsCheckBox_      ->setChecked( account->getNotifyContactsStatus()       );
  showOfflineNotificationsCheckBox_     ->setChecked( account->getNotifyContactsOffline()      );
  showInboxEmailNotificationsCheckBox_  ->setChecked( account->getNotifyInboxEmails()          );
  showOtherEmailNotificationsCheckBox_  ->setChecked( account->getNotifyOtherEmails()          );

  // Set the notification popups duration
  notificationDurationSpinBox_->setValue( account->getNotificationDuration() );

  // Also enable or disable the 'notify all incoming messages' when needed
  notificationsToggled();

  // Set the other alerts widgets
  idleTimeSpinBox_      ->setValue  ( account->getIdleTime()         );
  useIdleTimerCheckBox_ ->setChecked( account->getUseIdleTimer()     );
  shakeNudgeCheckBox_   ->setChecked( account->getShakeNudge()       );
  nowListeningCheckBox_ ->setChecked( account->getShowNowListening() );
  showEmailInfoCheckBox_->setChecked( account->getShowEmail()        );
}



/**
 * @brief Update the notifications options
 *
 * Enables or disables the "notify all incoming chat messages".
 */
void AlertsPage::notificationsToggled()
{
  showChatMessagesNotificationsCheckBox_->setEnabled( showChatStartNotificationsCheckBox_->isChecked() );
}



/**
 * @brief Save account information from the alerts widget
 *
 * Calls the set-up methods from Account to change its properties, based on how
 * the user has set the widgets of this page.
 */
void AlertsPage::saveSettings( Account *account )
{
  account->setAlertsInformation       ( useIdleTimerCheckBox_->isChecked(),
                                        idleTimeSpinBox_->value(),
                                        shakeNudgeCheckBox_->isChecked(),
                                        nowListeningCheckBox_->isChecked() );

  account->setNotificationsInformation( showOnlineNotificationsCheckBox_->isChecked(),
                                        showStatusNotificationsCheckBox_->isChecked(),
                                        showChatStartNotificationsCheckBox_->isChecked(),
                                        showChatMessagesNotificationsCheckBox_->isChecked(),
                                        showOfflineNotificationsCheckBox_->isChecked(),
                                        showInboxEmailNotificationsCheckBox_->isChecked(),
                                        showOtherEmailNotificationsCheckBox_->isChecked(),
                                        notificationDurationSpinBox_->value() );
}



// Force the page onto a specific tab
void AlertsPage::switchToTab( int tabIndex )
{
  if( tabIndex < 0 || tabIndex > tabWidget_->count() )
  {
    kWarning() << "Invalid tab index" << tabIndex << "selected!";
    return;
  }

  tabWidget_->setCurrentIndex( tabIndex );
}



#include "alertspage.moc"
