/***************************************************************************
                          kmessshared.h  -  description
                             -------------------
    begin                : Thu May 8 2008
    copyright            : (C) 2008 by Valerio Pilo
    email                : valerio@kmess.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMESSSHARED_H
#define KMESSSHARED_H

#include <QObject>


// Forward declaration
class KUrl;



/**
 * @brief Shared globally useful methods for KMess
 *
 * This class is just a container for many methods which are useful throughout the entire
 * application.
 *
 * @author Valerio Pilo <valerio@kmess.org>
 * @ingroup Utils
 */
class KMessShared
{

  public: // Public static methods
    // Create a HMACSha1 hash
    static QByteArray      createHMACSha1( const QByteArray &keyForHash, const QByteArray &secret );
    // Return a derived key with HMACSha1 algorithm
    static QByteArray      deriveKey( const QByteArray &keyToDerive, const QByteArray &magic );
    // Generate a random GUID.
    static QString         generateGUID();
    // Generate an random number to use as ID
    static unsigned long   generateID();
    // Return the hash of a file name using the SHA1 algorithm
    static QByteArray      generateFileHash( const QString &fileName );
    // Open the given URL respecting the user's preference
    static void            openBrowser( const KUrl &url );
    // Open the given URL mail respecting the user's preference
    static void            openEmailClient( const QString folder, bool isHotmailAvailable = true );
    // Open the give Profile
    static void            openProfile( const QString &handle );

  private:
    // Create the html file to request the given service
    static QString         createRequestFile( const QString &folder );
};

#endif
